Package: NanoTube
Type: Package
Title: An Easy Pipeline for NanoString nCounter Data Analysis
Version: 1.16.0
Date: 2023-05-31
Authors@R: c(person("Caleb", "Class", email="cclass@butler.edu",
                  role = c("cre", "aut"), 
                  comment = c(ORCID = "0000-0003-3130-3613")),
             person("Caiden", "Lukan", role = "ctb"))
Depends: R (>= 4.1), Biobase, ggplot2, limma
Imports: fgsea, methods, reshape, stats, utils
Suggests: grid, kableExtra, knitr, NanoStringDiff, pheatmap, plotly,
        rlang, rmarkdown, ruv, RUVSeq, shiny, testthat, xlsx
VignetteBuilder: knitr
Description: NanoTube includes functions for the processing, quality control, 
    analysis, and visualization of NanoString nCounter data. Analysis functions
    include differential analysis and gene set analysis methods, as well as
    postprocessing steps to help understand the results. Additional functions
    are included to enable interoperability with other Bioconductor NanoString
    data analysis packages.
License: GPL-3 + file LICENSE
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.2.1
biocViews: Software, GeneExpression, DifferentialExpression,
        QualityControl
git_url: https://git.bioconductor.org/packages/NanoTube
git_branch: RELEASE_3_22
git_last_commit: b78a412
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:23:17 UTC; biocbuild
Author: Caleb Class [cre, aut] (ORCID: <https://orcid.org/0000-0003-3130-3613>),
  Caiden Lukan [ctb]
Maintainer: Caleb Class <cclass@butler.edu>

# Generated by roxygen2: do not edit by hand

export(deVolcano)
export(fgseaPostprocessing)
export(fgseaPostprocessingXLSX)
export(fgseaToLEdge)
export(gm_mean)
export(groupFGSEA)
export(groupedGSEAtoStackedReport)
export(limmaToFGSEA)
export(makeDiffExprFile)
export(makeNanoStringSetFromEset)
export(nanostringPCA)
export(negativeQC)
export(normalize_housekeeping)
export(normalize_pos_controls)
export(nsdiffToFGSEA)
export(positiveQC)
export(processNanostringData)
export(read_cpdb_sourceDBs)
export(read_cpdb_tab)
export(read_merge_rcc)
export(read_rcc)
export(read_sampleData)
export(remove_background)
export(runLimmaAnalysis)
export(untar_dirs)
export(unzip_dirs)
import(Biobase)
import(fgsea)
import(ggplot2)
import(limma)
import(reshape)
importFrom(methods,is)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,untar)
importFrom(utils,unzip)
importFrom(utils,write.table)

# NanoTube

<a href="https://www.bioconductor.org/packages/NanoTube/">The NanoTube is on Bioconductor!</a>

NanoTube includes functions for the processing, quality control, 
analysis, and visualization of NanoString nCounter data. Analysis functions
include differential analysis and gene set analysis methods, as well as
postprocessing steps to help understand the results. Additional functions
are included to enable interoperability with other Bioconductor NanoString
data analysis packages.

Many more details can be found in the vignette (vignettes/NanoTube.Rmd).

If you use the NanoTube in your work, please cite:
Class CA, Lukan CJ, Bristow CA, Do KA. "Easy NanoString nCounter data analysis
with the NanoTube." Bioinformatics 2023; 39(1): btac762. https://doi.org/10.1093/bioinformatics/btac762

#' Postprocessing for GSEA analyses for Excel
#'
#' Clusters GSEA results by leading edge genes, and writes reports showing
#' gene expression profiles of these genes (to Excel).
#'
#' @export
#'
#' @param genesetResults Results from pathway analysis using limmaToFGSEA.
#' @param leadingEdge Results from fgseaToLEdge
#' @param limmaResults Results from runLimmaAnalysis
#' @param join.threshold The threshold distance to join gene sets. Gene sets 
#' with a distance below this value will be joined to a single "cluster."
#' @param ngroups The desired number of gene set groups. Either 
#' 'join.threshold' or 'ngroups' must be specified, 'ngroups' takes priority 
#' if both are specified.
#' @param dist.method Method for distance calculation (see options for dist()).
#' We recommend the 'binary' (also known as Jaccard) distance.
#' @param filename File name for the output Excel file.
#' 
#' @return An Excel file where the first sheet summarizes the gene set analysis 
#' results. Subsequent sheets are reports showing differential expression 
#' statistics of leading edge genes.
#' 
#' @examples
#' data("ExamplePathways")
#' data("ExampleResults") # Results from runLimmaAnalysis
#' 
#' fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways)
#' 
#' leadingEdge <- fgseaToLEdge(fgseaResults, cutoff.type = "padj", cutoff = 0.1)
#' 
#' \donttest{
#' fgseaPostprocessingXLSX(fgseaResults, leadingEdge, 
#'                     limmaResults = ExampleResults,
#'                     join.threshold = 0.5,
#'                     filename = "Results.xlsx")
#' }


fgseaPostprocessingXLSX <- function(genesetResults, leadingEdge, limmaResults,
                                    join.threshold = 0.5, ngroups = NULL,
                                    dist.method = "binary", filename) {

    tab <- makeFGSEAmasterTable(genesetResults, leadingEdge,
                                  join.threshold)
    xlsx::write.xlsx(tab, filename, sheetName="Summary",
               row.names = FALSE, col.names = TRUE, append = FALSE)
    
    for (i in names(genesetResults)) {
        if(!is.null(leadingEdge[[i]])) {
            grouped <-   groupFGSEA(genesetResults[[i]],
                                         leadingEdge[[i]],
                                         join.threshold,
                                         ngroups, dist.method,
                                         returns = "signif")
            
            tab <- groupedGSEAtoStackedReport(grouped, leadingEdge[[i]], 
                                                       limmaResults)
            xlsx::write.xlsx(tab, filename, sheetName=i,
                             row.names = FALSE, col.names = TRUE, append = TRUE)
        }
    }
  
}

#' Plot PCA
#'
#' Conduct principal components analysis and plot the results, using either
#' ggplot2 or plotly.
#'
#' @export
#'
#' @param ns Processed NanoString data
#' @param pc1 Principal component to plot on x-axis (default 1)
#' @param pc2 Principal component to plot on y-axis (default 2)
#' @param interactive.plot Plot using plotly? Default FALSE (in which case
#' ggplot2 is used)
#' @param exclude.zeros Exclude genes that are not detected in all samples
#' (default TRUE)
#' @param codeclass.retain The CodeClasses to retain for principal components 
#' analysis.Generally we're interested in endogenous genes, so we keep 
#' "endogenous" only by default. Others can be included by entering a character 
#' vector for this option. Alternatively, all targets can be retained by 
#' setting this option to ".". 
#' @return A list containing:
#' \item{pca}{The PCA object}
#' \item{plt}{The PCA plot}
#' 
#' @examples 
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' sample_data <- system.file("extdata", "GSE117751_sample_data.csv", 
#'                            package = "NanoTube")
#' 
#' # Process and normalize data first
#' dat <- processNanostringData(example_data, 
#'                              sampleTab = sample_data, 
#'                              groupCol = "Sample_Diagnosis",
#'                              normalization = "nSolver", 
#'                              bgType = "t.test", bgPVal = 0.01)
#'                                
#' # Interactive PCA using plotly                             
#' nanostringPCA(dat, interactive.plot = TRUE)$plt
#' 
#' # Static plot using ggplot2, for the 3rd and 4th PC's.
#' nanostringPCA(dat, pc1 = 3, pc2 = 4, interactive.plot = FALSE)$plt

nanostringPCA <- function(ns, pc1 = 1, pc2 = 2, 
                          interactive.plot = FALSE, 
                          exclude.zeros = TRUE,
                          codeclass.retain = "endogenous") {
  
    # Bind local variables
    PC1 <- PC2 <- group <- NULL
    
    # Convert codeclass.retain option to format that can be used by grep.
    codeclass.grep <- paste(codeclass.retain, collapse = "|")
    
    if (is(ns, "list")) {
        pca.dat <- ns$exprs[grep(codeclass.grep, ns$dict$CodeClass, ignore.case = TRUE),]
    } else {
        pca.dat <- exprs(ns)[grep(codeclass.grep, fData(ns)$CodeClass, ignore.case = TRUE),]
    }
    
    if (exclude.zeros) pca.dat <- pca.dat[rowSums(pca.dat == 0) == 0,]
    
    # If RUV normalization was used, data are already log-transformed.
    # Otherwise, log-transform with a pseudocount of 0.5
    if (ns$normalization[1] != "RUVIII") pca.dat <- log2(pca.dat + 0.5)
    
    pca <- prcomp(t(pca.dat),
                  center = TRUE, scale = TRUE)
    
    pca.ly <- as.data.frame(pca$x[,c(pc1, pc2)])
    colnames(pca.ly) <- c("PC1", "PC2")
    pca.ly$sample <- row.names(pca$x)
    pca.ly$group <- ns$groups
    
    # Percent of variation explained by each PC
    perc.var <- pca$sdev^2 / sum(pca$sdev^2) * 100
    
    if (interactive.plot) {
      
        layout.x <- layout.y <- list(
          showline = TRUE,
          showticklabels = TRUE,
          showgrid = FALSE,
          zeroline = FALSE,
          linecolor = plotly::toRGB("black"),
          linewidth = 1
        )
        
        layout.x$title <- paste0("PC", pc1, " (", 
                                 signif(perc.var[pc1], 2), "% var)")
        layout.y$title <- paste0("PC", pc2, " (", 
                                 signif(perc.var[pc2], 2), "% var)")
        
        `%>%` <- plotly::`%>%`
        
        plt <- plotly::plot_ly(data = pca.ly, x = ~PC1, y = ~PC2,
                               text = ~paste0("Sample: ", sample), 
                               color = ~group,
                               type = "scatter", mode = "markers") %>%
          plotly::layout(xaxis = layout.x, yaxis = layout.y)
    } else {
      
        plt <- ggplot(data = pca.ly,  
                      aes(x = PC1, y = PC2, color = group)) +
          geom_point() + theme_bw() +
          xlab(paste0("PC", pc1, " (", signif(perc.var[pc1], 2), "% var)")) +
          ylab(paste0("PC", pc2, " (", signif(perc.var[pc2], 2), "% var)"))
      
    }
  
    
    return(list(pca = pca, 
                plt = plt))
}

#' Make differential expression results file.
#'
#' Make a data frame or text file containing coefficients, p-, and q-values 
#' from Limma differential expression analysis. If returns == "all", will also 
#' center the log-expression data on the median of base.group expression, and 
#' include the expression data in the output.
#' 
#' @export
#' 
#' @param limmaResults Result from runLimmaAnalysis
#' @param filename The desired name for the output tab-delimited text file. If 
#' NULL (default) the resulting table will be returned as an R data frame.
#' @param returns If "all" (default), will center the log-expression
#' data on median of base.group expression and include the expression data in 
#' the output. If "stats", will only include the differential expression 
#' statistics.
#' @param skip.first Logical: Skip the first factor for gene set analysis?
#' Frequently the first factor is the 'Intercept', which is generally 
#' uninteresting for GSEA (default TRUE).
#' 
#' @return A table of differential expression results
#' 
#' @examples 
#' data("ExampleResults") # Results from runLimmaAnalysis
#' 
#' # Include expression data in the results table
#' deResults <- makeDiffExprFile(ExampleResults, returns = "all")
#' 
#' # Only include statistics, and save to a .txt file
#' \donttest{
#' makeDiffExprFile(ExampleResults, file = "DE.txt",
#'                  returns = "stats")
#' }

makeDiffExprFile <- function(limmaResults, filename = NULL,
                             returns = c("all", "stats"),
                             skip.first = TRUE) {
    
    # If groups are not provided, center by overall median.
    if(is.null(limmaResults$eset$group)) {
      dat.scaled <- exprs(limmaResults$eset) -
        apply(exprs(limmaResults$eset), 1, median)
    }
    else {  # If groups are provided, center by base group.
      dat.scaled <- exprs(limmaResults$eset) -
        apply(exprs(limmaResults$eset)[,limmaResults$eset$group == 
                                         levels(limmaResults$eset$group)[1]], 
              1, median)
      dat.scaled <- dat.scaled[,order(limmaResults$eset$group)]
    }
    
  
    limma.dat <- list()
  
    inc <- colnames(limmaResults$t)
    if (skip.first) inc <- inc[-1]
  
    for (i in inc) {
        limma.dat[[i]] <- cbind(limmaResults$coefficients[,i], 
                                limmaResults$t[,i], 
                                limmaResults$p.value[,i], 
                                limmaResults$q.value[,i])
        colnames(limma.dat[[i]]) <- 
          paste0(c("Log2FC (", "t (", "p-val (", "q-val ("), i, ")")
    }
    limma.tab <- do.call(cbind, limma.dat)
  
    if (returns == "stats") {
  
      limma.tab <- signif(limma.tab, digits = 2)
  
      if (!is.null(filename)) {
          dat.xpt <- cbind(rownames(limma.tab), limma.tab)
          colnames(dat.xpt)[1] <- "Symbol"
          write.table(dat.xpt, filename,
                      row.names = FALSE, col.names = TRUE, 
                      sep = "\t", quote = FALSE)
      } else {
          return(limma.tab)
      }
    } else {
  
        dat.xpt <- cbind(limma.tab, dat.scaled)
    
        if (!is.null(filename)) {
            dat.xpt <- cbind(rownames(dat.xpt), dat.xpt)
            colnames(dat.xpt)[1] <- "Symbol"
            write.table(dat.xpt, filename,
                        row.names = FALSE, col.names = TRUE, 
                        sep = "\t", quote = FALSE)
        } else {
            return(dat.xpt)
        }
    }

}

#' Run gene set enrichment analysis using DE results.
#' 
#' Use the fgsea library to run gene set enrichment analysis from the Limma
#' analysis results. Genes will be ranked by their log2 fold changes
#' or t-statistics (specified using 'rank.by').
#' 
#' Limma returns matrices of coefficients and t statistics with 
#' columns for each column in the design matrix. This function will conduct a
#' separate enrichment analysis on each column from the relevant matrix. Because
#' the first column may be an "intercept" term, which is generally not relevant
#' for enrichment analysis, the user may want to skip analysis for that term
#' (using skip.first = TRUE, the default).
#' 
#' @export
#' 
#' @param limmaResults Result from runLimmaAnalysis.
#' @param gene.sets Gene set file name, in .rds (list), .gmt, or .tab format;
#' or a list object containing the gene sets. Gene names must be
#' in the same form as in the ranked.list.
#' @param sourceDB Source database to include (only if using a .tab-format 
#' geneset.file from CPDB).
#' @param min.set Number of genes required to conduct analysis on a given gene 
#' set (default = 1). If fewer than this number of genes from limmaResults are 
#' included in a gene set, that gene set will be skipped for this analysis.
#' @param rank.by Rank genes by log2 fold changes ('coefficients', default) or 
#' t-statistics ('t').
#' @param skip.first Logical: Skip the first factor for gene set analysis?
#' Frequently the first factor is the 'Intercept', which is generally 
#' uninteresting for GSEA (default TRUE).
#' @return A list containing data frames with the fgsea results for each 
#' comparison. 
#' 
#' @examples 
#' data("ExamplePathways")
#' data("ExampleResults") # Results from runLimmaAnalysis
#' 
#' # Use the default settings
#' fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways)
#' 
#' # Only include gene sets with at least 5 genes in the NanoString data set,
#' # and rank genes by their "t" statistics.
#' fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways,
#'                              min.set = 5, rank.by = "t")

limmaToFGSEA <- function(limmaResults, gene.sets, sourceDB = NULL,
                         min.set = 1, rank.by = c('coefficients', 't'),
                         skip.first = TRUE) {

    if (is(gene.sets, "list")) {
        gene.set.list <- gene.sets
    } else {
        file.type <- substr(gene.sets, start = nchar(gene.sets)-2, 
                            stop = nchar(gene.sets))
        if (file.type == "rds") {
            gene.set.list <- readRDS(gene.sets)
        } else if (file.type == "gmt") {
            gene.set.list <- fgsea::gmtPathways(gene.sets)
        } else if (file.type == "tab") {
            gene.set.list <- read_cpdb_tab(gene.sets, sourceDB)
        } else {
            stop("gene.sets must be in .rds (list object), .gmt, 
                 or .tab format")
        }
    }
    
    rank.by <- rank.by[1]
    if (!(rank.by) %in% c('coefficients', 't')) {
        stop("rank.by must be either 'coefficeints' or 't'.")
    }
    
    if (skip.first) {
        analysis.factors <- colnames(limmaResults[[rank.by]])[-1]
    } else {
        analysis.factors <- colnames(limmaResults[[rank.by]])
    }
    
    fgsea.res <- list()
    for (i in analysis.factors) {
        fgsea.res[[i]] <- fgsea::fgseaMultilevel(pathways = gene.set.list,
                                          stats = limmaResults[[rank.by]][,i],
                                          minSize = min.set)
    }
    
    return(fgsea.res)
  
}
  

#' Merge multiple .rcc files
#'
#' Read in multiple .rcc files named in the fileList and merge the expression 
#' data. This step is conducted within processNanostringData.
#' 
#' @export
#'
#' @param fileList a character vector of .rcc file names
#' @param includeQC include merged QC data (from the "Lane Attributes" part of 
#' file) in the output? Default FALSE
#' @param logfile a filename for the logfile (optional). If blank, will print 
#' warnings to screen.
#' 
#' @return A list object including:
#' \item{exprs}{The expression matrix}
#' \item{dict}{The gene dictionary}
#' \item{qc}{QC metrics included in the .rcc files, if includeQC == TRUE}
#' 
#' @examples 
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' 
#' dat <- read_merge_rcc(list.files(example_data, full.names = TRUE))

read_merge_rcc <- function(fileList, includeQC = FALSE, logfile = "") {
    dat <- lapply(fileList, read_rcc)
  
    cat("\nChecking codeset contents......", file=logfile, append=TRUE)
    
    # Check that all files have same gene names in same order
    # We could allow merging of discordant dictionaries in a future update.
    # Otherwise, these can be read separately and then merged.
    if (length(unique(lapply(dat, function(df) df$exprs$Name))) > 1) {
        stop(cat("\nRCC files do not have same gene names. Stopping...", 
                 file=logfile, append=TRUE))
    }
    cat("\nChecked gene name consistency in .RCC files.", 
        file=logfile, append=TRUE)
  
    vec.template <- dat[[1]][[1]][,4]
    merged.dat <- list(exprs = vapply(dat, function(X) X[[1]][,4],
                                      FUN.VALUE = vec.template),
                       dict = dat[[1]][[1]][,seq_len(3)])
  
    # We use Accessions for row names, but it's possible for those to be 
    # duplicated. To account for this, use the `make.names` function.
    rownames(merged.dat[[1]]) <- rownames(merged.dat[[2]]) <- 
      make.names(merged.dat$dict[,3], unique = TRUE)
    colnames(merged.dat[[1]]) <- gsub(".*\\/", "", fileList)
  
    if (any(merged.dat$dict$CodeClass == "Housekeeping")) {
        cat("\n", paste(sum(merged.dat$dict$CodeClass == "Housekeeping")), 
            " genes labeled as 'housekeeping'.",
            file=logfile, sep = "", append = TRUE)
    } else {
        warning(cat("\nNo genes labeled as 'housekeeping' in data set.", 
                    file=logfile, append=TRUE))
    }
  
    if (includeQC == TRUE) {
        vec.template <- dat[[1]][[3]][,2]
        merged.dat$qc <-  cbind(dat[[1]][[3]][,1], 
                                vapply(dat, function(X) X[[3]][,2],
                                       FUN.VALUE = vec.template))
        rownames(merged.dat$qc) <- merged.dat$qc[,1]
        merged.dat$qc <- t(merged.dat$qc[,-1])
        rownames(merged.dat$qc) <- colnames(merged.dat$exprs)
    }
    return(merged.dat)
}

#' Calculate negative control statistics
#'
#' Provide a table the negative control statistics, and plot the counts of 
#' negative control genes in each sample.
#' 
#' @export
#'
#' @param ns NanoString data, processed by `processNanostringData` with
#' output.format set to 'list' and 'nSolver' normalization.
#' @param interactive.plot Generate an interactive plot using plotly? Only 
#' recommended for fewer than 20 samples (default FALSE)
#' @return A list object containing:
#' \item{tab}{The table of negative control statistics, including the mean
#' & standard deviation of negative control genes, calculated background
#' threshold, and number of endogenous genes below that threshold}
#' \item{plt}{An object containing the negative control plots.}
#' 
#' @examples
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' sample_data <- system.file("extdata", "GSE117751_sample_data.csv", 
#'                            package = "NanoTube")
#' 
#' # Process and normalize data first
#' dat <- processNanostringData(example_data, 
#'                              sampleTab = sample_data, 
#'                              groupCol = "Sample_Diagnosis",
#'                              normalization = "nSolver", 
#'                              bgType = "threshold", 
#'                              bgThreshold = 2, bgProportion = 0.5,
#'                              output.format = "list")
#' 
#' negQC <- negativeQC(dat, interactive.plot = FALSE) 
#' 
#' # View negative QC table & plot
#' head(negQC$tab)
#' negQC$plt

negativeQC <- function(ns, interactive.plot = FALSE) {
  
    if (ns$normalization != "nSolver") {
        stop("Must run processNanostringData with normalization = 'nSolver' 
             prior to using this function")
    }
    
    if (!is(ns, "list")) {
        stop("Must run processNanostringData with output.format = 'list' prior
             to using this function")
    }
    
    # Bind local variables
    Count <- Sample <- Gene <- NULL
    
    # Strip plot for negative control genes
    dat.neg <- as.data.frame(ns$exprs.raw[ns$dict.raw$CodeClass == "Negative",])
    dat.neg$Gene <- ns$dict.raw$Name[ns$dict.raw$CodeClass == "Negative"]
    
    dat.neg.df <- reshape::melt(dat.neg, "Gene")
    colnames(dat.neg.df)[2:3] <- c("Sample", "Count")
    
    
    # Negative Table
    if (ncol(ns$bg.stats) == 3) {
        neg.tab <- round(ns$bg.stats, 2)
    } else {
        neg.tab <- round(ns$bg.stats[,seq_len(4)], 2)
        neg.tab$fail <- paste0(ns$bg.stats$num.less.bg, " (",
                               round(ns$bg.stats$frc.less.bg*100, 1), "%)")
        colnames(neg.tab) <- c("Mean (Neg)", "Max (Neg)", "sd (Neg)", 
                               "Background cutoff", 
                               "Genes below BG (%)")
    }
    
    neg.plot <- ggplot(data = dat.neg.df, 
                       aes(x=Count, y=Sample, 
                           text=paste0("Sample: ", Sample, "\nGene: ", Gene, 
                                       "\nCount: ", Count))) +
      geom_jitter(height = 0.2, width = 0, colour = "black", 
                  fill = "grey70", pch=21) +
      theme_classic() + ylab("") 
    
    if (interactive.plot) {
        neg.plot <- plotly::ggplotly(neg.plot, tooltip = c("text"), 
                                     width = 550, height = 400)
    }
    
    return(list(tab = neg.tab,
                plt = neg.plot))
}

#' Build a report from gene set enrichment results.
#' 
#' After clustering FGSEA results by gene set similarity, this function builds
#' a report containing the individual gene expression profiles for genes 
#' contained in each gene set cluster.
#' 
#' @export
#' 
#' @param grouped.gsea Output from groupFGSEA()
#' @param leadingEdge Leading edge analysis results used in groupFGSEA()
#' @param de.fit Differential Expression results from Limma or NanoStringDiff
#' @param outputDir Directory for output files. If NULL (default), will return
#' the stacked report instead of writing to a file.
#' 
#' @return A stacked report containing statistics and gene expression profiles
#' for genes contained in each cluster
#' 
#' @examples
#' data("ExamplePathways")
#' data("ExampleResults") # Results from runLimmaAnalysis
#' 
#' fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways,
#'                              min.set = 5, rank.by = "t")
#' leadingEdge <- fgseaToLEdge(fgseaResults, cutoff.type = "padj", cutoff = 0.1)
#' 
#' fgseaGrouped <- groupFGSEA(fgseaResults$Autoimmune.retinopathy, 
#'                             leadingEdge$Autoimmune.retinopathy,
#'                             join.threshold = 0.5,
#'                             dist.method = "binary")
#' 
#' results.AR <- groupedGSEAtoStackedReport(
#'               fgseaGrouped,
#'               leadingEdge = leadingEdge$Autoimmune.retinopathy,
#'               de.fit = ExampleResults)

groupedGSEAtoStackedReport <- function(grouped.gsea, leadingEdge, de.fit,
                                       outputDir=NULL) {
  
    diffExpr.mat <- makeDiffExprFile(de.fit, returns = "all")
    Cluster <- rep(1, times = nrow(diffExpr.mat))
    names(Cluster) <- rownames(diffExpr.mat)
    diffExpr.mat <- cbind(Cluster, diffExpr.mat)
    
    grouped.gsea <-
      grouped.gsea[rownames(grouped.gsea) %in% colnames(leadingEdge), ]
    
    xpt.mat.full <- merge(diffExpr.mat,
                          leadingEdge[, as.character(rownames(grouped.gsea))],
                          by = "row.names", all = FALSE)
    colnames(xpt.mat.full)[1] <- "Symbol"
    
    if (ncol(leadingEdge) == 1) {
        xpt.mat <- xpt.mat.full[xpt.mat.full$y == 1,]
        colnames(xpt.mat)[ncol(xpt.mat)] <- rownames(grouped.gsea)
    } else {
        xpt.mat <- matrix(nrow=0, ncol=ncol(xpt.mat.full))
        colnames(xpt.mat) <- colnames(xpt.mat.full)
    
        grouped.gsea <- cbind(as.character(rownames(grouped.gsea)), 
                              grouped.gsea)
        colnames(grouped.gsea)[1] <- 'gene set'
      
      for (i in seq_len(max(grouped.gsea$Cluster))) {
          grouped.cluster <- grouped.gsea[grouped.gsea$Cluster == i,]
          xpt.mat.full[,2] <- i
          
          leadingCluster <- xpt.mat.full[,grouped.cluster$`gene set`]
          
          if(is.null(dim(leadingCluster))) {
            xpt.mat <- rbind(xpt.mat,
                             xpt.mat.full[leadingCluster == 1,])
          } else {
            xpt.mat <- rbind(xpt.mat,
                             xpt.mat.full[rowSums(leadingCluster) >= 1,])
          }
      }
    }
    
    if (!is.null(outputDir)) {
        dir.create(outputDir)
        write.table(grouped.gsea, file=paste0(outputDir, "/summaryReport.txt"),
                    col.names=TRUE, row.names=FALSE, sep = "\t", quote=FALSE)
        write.table(xpt.mat, file=paste0(outputDir, "/fullResults.txt"),
                    col.names=TRUE, row.names=FALSE, sep = "\t", quote=FALSE)
    } else {
        return(xpt.mat)
    }
}

#' NanoTube
#' 
#' A package for NanoString nCounter gene expression data processing,
#' analysis, and visualization.
#' 
#' @name NanoTube
#' @docType package
#' @import Biobase ggplot2 fgsea limma reshape
#' @importFrom stats cor cutree dist hclust median model.matrix p.adjust 
#'             prcomp sd t.test
#' @importFrom utils read.csv read.delim unzip untar write.table
#' @importFrom methods is
NULL

#' Example pathway database
#' 
#' A list object containing example gene sets from WikiPathways.
#' 
#' @name ExamplePathways
#' @docType data
#' @keywords datasets
#' @usage data(ExamplePathways)
#' @format A list object with 30 vectors of gene symbols, for 30 pathways
NULL

#' Example results from runLimmaAnalysis
#' 
#' Results of runLimmaAnalysis using the example data set GSE117751 
#' (in extdata).
#' 
#' @name ExampleResults
#' @docType data
#' @keywords datasets
#' @usage data(ExampleResults)
#' @format An MArrayLM object from limma
NULL

#' Read .tab file.
#'
#' Read in a .tab file from the Consensus Pathway Database (CPDB)
#' 
#' @export
#'
#' @param file The filename
#' @param sourceDB The source database to use. If NULL (default), retains
#' gene sets from all source databases
#'
#' @return A list object, containing a character vector of genes for
#' each gene set.

read_cpdb_tab <- function(file, sourceDB = NULL) {
    if (!grepl("\\.tab$", file)[1]) {
        stop("Pathway information must be a .tab file")
    }
    
    geneSetTable <- read.delim(file,
                               stringsAsFactors = FALSE)
    
    if (!is.null(sourceDB)) {
        if (!(sourceDB %in% geneSetTable$source)) {
            stop("'", sourceDB, 
                 "' is not one of the source databases in ", 
                 file)
        }
      
        geneSetTable <- geneSetTable[geneSetTable$source == sourceDB,]
    }
    
    geneSetDB <- strsplit(geneSetTable[,ncol(geneSetTable)], split = ",")
    
    if (is.null(sourceDB)) {
        names(geneSetDB) <- paste0(geneSetTable$pathway, " | ", geneSetTable$source)
    } else {
        names(geneSetDB) <- geneSetTable$pathway
    }
    
    return(geneSetDB)
}

#' Convert NanoString ExpressionSet to NanoStringSet
#'
#' Convert ExpressionSet from processNanoStringData to a NanoStringSet for use
#' with the NanoStringDiff package.
#' 
#' @export
#'
#' @param eset NanoString data ExpressionSet, from processNanostringData
#' @param designs Design matrix. If NULL, will look for "groups" column in 
#' pData(eset).
#' @return A NanoStringSet for NanoStringDiff
#' 
#' @examples 
#' # Example data
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' sample_data <- system.file("extdata", "GSE117751_sample_data.csv", 
#' package = "NanoTube")
#' 
#' # Load data without normalization
#' dat <- processNanostringData(nsFiles = example_data,
#'                      sampleTab = sample_data, groupCol = "Sample_Diagnosis",
#'                      normalization = "none")
#'                      
#' # Convert to NanoStringSet
#' dat.ns <- makeNanoStringSetFromEset(dat)

makeNanoStringSetFromEset <- function(eset, designs = NULL) {
  
    # Order positive control data (required for NanoStringDiff)
    pos.unordered <- eset[grep("positive", fData(eset)$CodeClass, ignore.case = TRUE),]
    pos.ordered <- exprs(pos.unordered)[order(fData(pos.unordered)$Name),]
    
    # Look for group info
    if (is.null(designs)) {
        if ("groups" %in% colnames(pData(eset))) {
            designs <- model.matrix(~0 + eset$groups)
            # Clean up column names
            colnames(designs) <- gsub("eset\\$groups", "", colnames(designs))
        } else {
            stop("Must input design matrix, or include groups 
                 in sample info of eset.")
        }
    }
    
    # Build NanoStringSet
    nsSet <- NanoStringDiff::createNanoStringSet(
        endogenous = exprs(eset)[grep("endogenous", fData(eset)$CodeClass, ignore.case = TRUE),],
        positiveControl = pos.ordered,
        negativeControl = exprs(eset)[grep("negative", fData(eset)$CodeClass, ignore.case = TRUE),],
        housekeepingControl = exprs(eset)[
          grep("housekeeping", fData(eset)$CodeClass, ignore.case = TRUE),],
        designs = designs)
    
    fData(nsSet) <- fData(eset)[grep("endogenous", fData(eset)$CodeClass, ignore.case = TRUE),]
    rownames(nsSet) <- fData(nsSet)$Name
    
    return(nsSet)
}

#' Housekeeping gene normalization
#'
#' Scale endogenous and housekeeping genes by the geometric mean of 
#' housekeeping genes. This should be conducted after positive control 
#' normalization and background correction.  This step is conducted 
#' within processNanostringData, when normalization is set to "nCounter".
#'
#' @export
#' 
#' @param dat NanoString data, including expression matrix and gene dictionary.
#' @param genes List of housekeeping genes to use for normalization. If NULL 
#' (default), will use all genes marked as "Housekeeping" in codeset.
#' @param logfile Optional name of logfile to print messages, warnings 
#' or errors.
#' 
#' @return NanoString data, with expression matrix now normalized by 
#' housekeeping gene expression.
#' 
#' @examples
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' 
#' # Load data, positive control normalization, and background filtering
#' dat <- read_merge_rcc(list.files(example_data, full.names = TRUE))
#' dat <- normalize_pos_controls(dat)
#' dat <- remove_background(dat, mode = "t.test", pval = 0.05)
#' 
#' # Normalize by genes marked "Housekeeping" in RCC files
#' dat <- normalize_housekeeping(dat)
#' 
#' # Normalize by specified housekeeping genes (gene symbol or accession)
#' dat <- normalize_housekeeping(dat,
#'                        genes = c("TUBB", "TBP", "POLR2A", "GUSB", "SDHA"))

normalize_housekeeping <- function(dat, genes = NULL, logfile = "") {

    if (is.null(genes)) genes <- 
        dat$dict$Name[grep("housekeeping", dat$dict$CodeClass, ignore.case = TRUE)]
  
    exprs.dat <- dat$exprs
  
    if (all(genes %in% dat$dict$Name)) {
        laneGM <- apply(exprs.dat[dat$dict$Name %in% genes,], 2, gm_mean)
    } else if (all(genes %in% dat$dict$Accession)) {
        laneGM <- apply(exprs.dat[dat$dict$Accession %in% genes,], 2, gm_mean)
    } else {
        stop(cat("\nHousekeeping gene(s) not found in dataset:", 
                 genes[!(genes %in% dat$dict$Name | 
                           genes %in% dat$dict$Accession)],
                 "\n", file=logfile, append=TRUE))
    }
  
    scale.factor <- laneGM / mean(laneGM)
    exprs.dat[grep("endogenous|housekeep", dat$dict$CodeClass, ignore.case=TRUE),] <-
      sweep(exprs.dat[grep("endogenous|housekeep", dat$dict$CodeClass, ignore.case=TRUE),], 
            2, scale.factor, '/')
    dat$exprs <- exprs.dat
    dat$hk.scalefactors <- scale.factor
    return(dat)
}

#' Assess background expression
#'
#' Compare endogenous gene expression data against negative control genes and
#' remove data for genes that fail the comparison. This step is
#' conducted within processNanostringData, when normalization is set to 
#' "nCounter".
#' 
#' @export
#'
#' @param dat Positive control-scaled NanoString data
#' @param mode Either "threshold" (default) or "t.test". If "threshold", 
#' requires proportionReq of samples to have expression numSD standard 
#' deviations among the mean of negative control genes. If "t.test", each gene 
#' will be compared with all negative control genes in a one-sided two-sample 
#' t-test. 
#' @param numSD Number of standard deviations above mean of negative control 
#' genes to used as background threshold for each sample: 
#' mean(negative_controls) + numSD * sd(negative_controls). 
#' Required if mode == "threshold" or subtract == TRUE
#' @param proportionReq Required proportion of sample expressions exceeding the
#' sample background threshold to include gene in further analysis. Required if
#' mode == "threshold" or subtract == TRUE
#' @param pval p-value (from one-sided t-test) threshold to declare gene 
#' expression above background expression level. Genes with p-values above 
#' this level are removed from further analysis. Required if mode == "t.test"
#' @param subtract Should calculated background levels be subtracted from 
#' reported expressions? If TRUE, will subtract mean+numSD*sd of the negative 
#' controls from the endogenous genes, and then set negative values to zero 
#' (default FALSE).
#' 
#' @return NanoString data, with genes removed that fail the comparison test
#' against negative control genes. Expression levels are updated for all genes
#' if subtract == TRUE.
#' 
#' @examples
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' 
#' # Load data and positive control normalization
#' dat <- read_merge_rcc(list.files(example_data, full.names = TRUE))
#' dat <- normalize_pos_controls(dat)
#' 
#' # Remove endogenous genes that fail to reject the null hypothesis
#' # in a one-sided t test against negative control genes with p < 0.05.
#' dat <- remove_background(dat, mode = "t.test", pval = 0.05)
#' 
#' # Remove endogenous genes where fewer than 25% of samples have an expression
#' # 2 standard deviations above the average negative control gene. Also, 
#' # subtract this background level (mean + 2*sd) from endogenous genes.
#' dat <- remove_background(dat, mode = "threshold", 
#'                          numSD = 2, proportionReq = 0.25, subtract = TRUE)

remove_background <- function(dat, 
                              mode = c("threshold", "t.test"), 
                              numSD, proportionReq, pval,
                              subtract = FALSE) {
  
    # Check for negative control genes in the data set
    if (sum(dat$dict$CodeClass == "Negative") == 0) {
        stop("Cannot conduct filtering with negative controls: No negative 
            control genes found in input")
    }
    
    negative.mean <- apply(dat$exprs[grep("negative", dat$dict$CodeClass, 
                                          ignore.case = TRUE),], 
                           2, mean)
    negative.sd <- apply(dat$exprs[grep("negative", dat$dict$CodeClass, 
                                        ignore.case = TRUE),], 2, sd)
    
    if (mode == "threshold" | subtract) {
        bg.threshold <- negative.mean + numSD * negative.sd
        
        dat$bg.stats <- data.frame(Mean.Neg = negative.mean,
                                   Max.Neg =
                                     apply(dat$exprs[
                                       grep("negative", dat$dict$CodeClass, 
                                            ignore.case = TRUE), ],
                                           2, max), 
                                   sd.Neg = negative.sd,
                                   background = bg.threshold,
                                   num.less.bg = 
                                     colSums(t(t(dat$exprs[
                                       grep("endogenous", dat$dict$CodeClass, 
                                            ignore.case = TRUE) , ]) <
                                                 bg.threshold)), 
                                   frc.less.bg = 
                                     colMeans(t(t(dat$exprs[
                                       grep("endogenous", dat$dict$CodeClass, 
                                            ignore.case = TRUE) , ]) <
                                                  bg.threshold)))
    } else {
        dat$bg.stats <- data.frame(Mean.Neg = negative.mean,
                                   Max.Neg = apply(dat$exprs[
                                     grep("negative", dat$dict$CodeClass, 
                                          ignore.case = TRUE),], 
                                     2, max),
                                   sd.Neg = negative.sd)
    }
    
    if (mode == "t.test") {
        dat$gene.stats <- data.frame(row.names = dat$dict$Name,
                                     t.stat = rep(NA, 
                                                  times=length(dat$dict$Name)),
                                     p.val = rep(NA, 
                                                 times=length(dat$dict$Name)),
                                     pass = rep(NA, 
                                                times=length(dat$dict$Name)))
        
        rowsKeep <- ifelse(grepl("endogenous", dat$dict$CodeClass, 
                                ignore.case = TRUE), 
                           yes = FALSE, no = TRUE)
        background <- unlist(dat$exprs[grep("negative", dat$dict$CodeClass, 
                                            ignore.case = TRUE),])
        
        for (i in grep("endogenous", dat$dict$CodeClass, 
                             ignore.case = TRUE)){
            ttest <- t.test(x = dat$exprs[i,], y = background, 
                            var.equal = FALSE, alternative = "greater")
            if (ttest$p.value < pval) {
                rowsKeep[i] <- TRUE
            }
            dat$gene.stats$t.stat[i] <- as.numeric(ttest$statistic)
            dat$gene.stats$p.val[i] <- ttest$p.value
        }
        
        dat$gene.stats$pass <- rowsKeep
    } else {
        rowsKeep <- which(!grepl("endogenous", dat$dict$CodeClass, 
                                 ignore.case = TRUE) | 
                            rowMeans(t(t(dat$exprs) > bg.threshold)) >= 
                            proportionReq)
    }
    
    if (subtract) {
        for (i in seq_len(ncol(dat$exprs))) dat$exprs[,i] <- 
            dat$exprs[,i] - bg.threshold[i]
        dat$exprs[dat$exprs < 0] <- 0
    }
    
    dat$exprs <- dat$exprs[rowsKeep,]
    dat$dict <- dat$dict[rowsKeep,]
    return(dat)
}

#' Run gene set enrichment analysis using DE results.
#' 
#' Use the fgsea library to run gene set enrichment analysis from the 
#' NanoStringDiff analysis results. Genes will be ranked by their log2 fold
#' changes.
#' 
#' @export
#' 
#' @param deResults Result from NanoStringDiff::glm.LRT.
#' @param gene.sets Gene set file name, in .rds (list), .gmt, or .tab format;
#' or a list object containing the gene sets. Gene names must be
#' in the same form as in the ranked.list.
#' @param sourceDB Source database to include, only if using a .tab-format 
#' geneset.file from CPDB.
#' @param min.set Number of genes required to conduct analysis on a given gene 
#' set (default = 1). If fewer than this number of genes from limmaResults are 
#' included in a gene set, that gene set will be skipped for this analysis.
#' @return A list containing data frames with the fgsea results.
#' 
#' @examples 
#' 
#' \donttest{
#'  
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' sample_data <- system.file("extdata", "GSE117751_sample_data.csv", 
#'                            package = "NanoTube")
#' 
#' datNoNorm <- processNanostringData(nsFiles = example_data,
#'                                    sampleTab = sample_data, 
#'                                    groupCol = "Sample_Diagnosis",
#'                                    normalization = "none")
#'
#' # Convert to NanoString Set, retaining 2 samples per group for this example
#' # (will run faster, but still pretty slow)
#' nsDiffSet <- makeNanoStringSetFromEset(datNoNorm[,c(1,2,15,16,29,30)])
#' 
#' # Run NanoStringDiff analysis
#' nsDiffSet <- NanoStringDiff::estNormalizationFactors(nsDiffSet)
#' result <- NanoStringDiff::glm.LRT(nsDiffSet, 
#'                                   design.full = as.matrix(pData(nsDiffSet)),
#'                                   contrast = c(1, -1, 0)) 
#'                                   #contrast: Autoimmune retinopathy vs. None
#' 
#' # FGSEA with example pathways, only for pathways with at least 5 genes
#' # analyzed in NanoString experiment
#' data("ExamplePathways")
#' fgseaResult <- nsdiffToFGSEA(result, gene.sets = ExamplePathways,
#'                              min.set = 5)
#' 
#' 
#' }

nsdiffToFGSEA <- function(deResults, gene.sets, sourceDB = NULL,
                         min.set = 1) {
  
    if (is(gene.sets, "list")) {
        gene.set.list <- gene.sets
    } else {
        file.type <- substr(gene.sets, start = nchar(gene.sets)-2, 
                            stop = nchar(gene.sets))
        if (file.type == "rds") {
            gene.set.list <- readRDS(gene.sets)
        } else if (file.type == "gmt") {
            gene.set.list <- fgsea::gmtPathways(gene.sets)
        } else if (file.type == "tab") {
            gene.set.list <- read_cpdb_tab(gene.sets, sourceDB)
        } else {
            stop("gene.sets must be in .rds (list object), .gmt, or 
                 .tab format")
        }
    }
  
    de.stats <- deResults$table$logFC
    names(de.stats) <- rownames(deResults$table)
    
    fgsea.res <- list(res = fgsea::fgseaMultilevel(pathways = gene.set.list,
                                                   stats = de.stats,
                                                   minSize = min.set))
    
    return(fgsea.res)
  
}

#' Calculate positive control statistics
#'
#' Calculate the linearity and scale factors of positive control genes, and
#' plot the expected vs. observed counts for each sample.
#' 
#' @export
#'
#' @param ns NanoString data, processed by `processNanostringData` with
#' normalization set to 'none' or with output.format set to 'list'.
#' @param samples A subset of samples to analyze (either a vector of sample 
#' names, or column indexes). If NULL (default), will include all samples.
#' @param expected The expected values of each positive control gene, as a 
#' numeric vector. These are frequently provided by NanoString in the 'Name' 
#' field of the genes, in which case those values will be read automatically 
#' and this option can be left as NULL (the default).
#' @return A list object containing:
#' \item{tab}{The table of positive control statistics, included the positive
#' scale factor and the R-squared value for the expected vs. measured counts}
#' \item{plt}{An object containing the positive control plots. This gets
#' cumbersome if there are lots of samples.}
#' 
#' @examples
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' sample_data <- system.file("extdata", 
#'                            "GSE117751_sample_data.csv", 
#'                            package = "NanoTube")
#' 
#' # Process data first. Must be output as a "list" or without normalization to
#' # obtain positive control statistics
#' dat <- processNanostringData(example_data, 
#'                              sampleTab = sample_data, 
#'                              groupCol = "Sample_Diagnosis",
#'                              normalization = "nSolver", 
#'                              bgType = "t.test", 
#'                              bgPVal = 0.01,
#'                              output.format = "list")
#' 
#' # Generate positive QC metrics for all samples
#' posQC <- positiveQC(dat) 
#' 
#' # View positive QC table & plot
#' head(posQC$tab)
#' posQC$plt
#' 
#' # Plot for only the first three samples
#' posQC <- positiveQC(dat, samples = 1:3)
#' posQC$plt

positiveQC <- function(ns, samples = NULL, expected = NULL) {
  
    # Bind local variables
    Expected <- Observed <- NULL
    
    # Log-transform the expected values if provided.
    if (!is.null(expected)) {
        expected <- log2(expected)
    }
    
    if (is(ns, "list")) {
        # For scale factors
        dat.sf <- list(exprs = ns$exprs.raw,
                       dict = ns$dict.raw)
      
        # For R-squared calculation
        dat.pos <-
            as.data.frame(log2(ns$exprs.raw[
                ns$dict.raw$CodeClass == "Positive", ]))
        
        # Identify the expected value for each positive control
        if (is.null(expected)) {
          expected <-
              log2(as.numeric(
                  gsub(".*\\(|\\)", "", 
                       ns$dict.raw$Name[ns$dict.raw$CodeClass == "Positive"])))
        }
      
    } else {
        if (ns$normalization[1] != "none") {
            stop("Raw NanoString data must be provided. Use 
                 normalization='none' or output.format='list' in 
                 `processNanostringData`.")
        }
        # For scale factors
        dat.sf <- list(exprs = exprs(ns),
                       dict = fData(ns))
      
        # For R-squared calculation
        dat.pos <- as.data.frame(log2(exprs(ns)[
            fData(ns)$CodeClass == "Positive",]))
        
        # Identify the expected value for each positive control
        if (is.null(expected)) {
            expected <- 
                log2(as.numeric(
                    gsub(".*\\(|\\)", "", 
                        fData(ns)$Name[fData(ns)$CodeClass == "Positive"])))
        }
    }
    
    
    # Use a subset of samples, if indicated
    if (!is.null(samples)) {
        dat.pos <- dat.pos[,samples]
    }
    
    # Calculate positive scale factors
    scale.factor <- normalize_pos_controls(dat = dat.sf)$pc.scalefactors
    
    # Calculate r-squared for each sample
    rsq <- vapply(dat.pos, function(vec) cor(vec, expected) ^ 2,
                  FUN.VALUE = 0)
    
    dat.pos$Expected <- expected
    
    dat.pos.df <- reshape::melt(dat.pos, "Expected")
    colnames(dat.pos.df)[2:3] <- c("Sample", "Observed")
    
    samps <- unique(dat.pos.df$Sample)
    xmin <- min(dat.pos.df$Expected)
    xmax <- max(dat.pos.df$Expected)
    ymin <- min(dat.pos.df$Observed)
    ymax <- max(dat.pos.df$Observed)
    
    pos.height <- (ncol(dat.pos)-1) * 2/3
    
    # Scale factor table
    pos.tab <- data.frame(Sample = colnames(dat.pos)[-ncol(dat.pos)],
                          Scale.Factor = scale.factor,
                          R.squared = rsq)
    
    
    pos.plot <- ggplot(data=dat.pos.df, aes(x=Expected, y=Observed)) +
      geom_point(colour = "black", fill = "grey70", pch=21, size=3) + 
      facet_wrap(~ Sample, ncol = 3) + 
      xlim(xmin, xmax) + ylim(ymin, ymax) + theme_bw()
    
    return(list(tab = pos.tab,
                plt = pos.plot))
}

#' Generate leading edge matrix from fgsea results.
#' 
#' Extract leading edge genes from gene sets identified in fgsea analysis.
#' Gene sets may be filtered by significance or NES.
#' 
#' @export
#' 
#' @param fgsea.res Result from limmaToFGSEA
#' @param cutoff.type Filter gene sets by adjusted p-value ('padj'), nominal
#' p-value ('pval'), normalized enrichment score ('NES'), or include all gene
#' sets ('none')
#' @param cutoff Numeric cutoff for filtering (not used if 
#' cutoff.type == "none")
#' @param nes.abs.cutoff If cutoff.type == "NES", should we use extreme positive
#' and negative values (TRUE), or only filter in the positive or negative 
#' direction (FALSE). If TRUE, will select gene sets with abs(NES) > cutoff. 
#' If FALSE, will select gene sets with NES > cutoff (if cutoff >= 0) or
#' NES < cutoff (if cutoff < 0)
#' @return a list containing the leading edge matrix for each comparison
#' 
#' @examples 
#' data("ExamplePathways")
#' data("ExampleResults") # Results from runLimmaAnalysis
#' 
#' fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways)
#' 
#' # Generate the leading edge for pathways with padj < 0.25
#' leadingEdge <- fgseaToLEdge(fgseaResults, 
#'                             cutoff.type = "padj", cutoff = 0.25)
#' 
#' # Generate the leading edge for pathways with abs(NES) > 2
#' leadingEdge <- fgseaToLEdge(fgseaResults, cutoff.type = "NES",
#'                             cutoff = 2, nes.abs.cutoff = TRUE)

fgseaToLEdge <- function(fgsea.res, 
                         cutoff.type = c("padj", "pval", "NES", "none"),
                         cutoff = 0.05, nes.abs.cutoff = TRUE) {
  
    # Check cutoff
    cutoff.type <- cutoff.type[1]
    if (cutoff.type %in% c("padj", "pval")) {
        if (cutoff <= 0) {
            stop("cutoff cannot be negative for 'pval' or 'padj'.")
        }
    } else if (!(cutoff.type %in% c("padj", "pval", "NES", "none"))) {
        stop("cutoff.type must be 'padj', 'pval', 'NES', or 'none'.")
    }
    
    ledge.res <- list()
    
    for (i in names(fgsea.res)) {
        fgsea.res[[i]] <- as.data.frame(fgsea.res[[i]])
        
        if (cutoff.type %in% c("padj", "pval")) {
            fgsea.sub <- 
              fgsea.res[[i]][as.vector(fgsea.res[[i]][,cutoff.type] < cutoff & 
                                         !is.na(fgsea.res[[i]][,cutoff.type])),]
        } else if (cutoff.type == "NES") {
            if (nes.abs.cutoff) {
                fgsea.sub <- fgsea.res[[i]][abs(fgsea.res[[i]][,"NES"]) > 
                                              cutoff & 
                                            !is.na(fgsea.res[[i]][,"NES"]),]
            } else if (cutoff >= 0) {
                fgsea.sub <- fgsea.res[[i]][fgsea.res[[i]][,"NES"] > cutoff & 
                                            !is.na(fgsea.res[[i]][,"NES"]),]
            } else {
                fgsea.sub <- fgsea.res[[i]][fgsea.res[[i]][,"NES"] < cutoff & 
                                            !is.na(fgsea.res[[i]][,"NES"]),]
            }
        } else {
            fgsea.sub <- fgsea.res[[i]]
        }
        
        if (nrow(fgsea.sub) == 0) {
            # Returning a NULL value in place of a leading edge matrix
            ledge.res[[i]] <- ""
            ledge.res[i] <- list(NULL)
            warning("No gene sets meeting cutoff. Will return empty leading 
                    edge matrix")
        } else {
            ledge.dat <- fgsea.sub$leadingEdge
            ledge.genes <- unique(unlist(ledge.dat))
            
            ledge.res[[i]] <- matrix(0, nrow = length(ledge.genes), 
                                     ncol = nrow(fgsea.sub),
                                     dimnames = list(ledge.genes, 
                                                     fgsea.sub$pathway))
            
            for (j in seq_len(nrow(fgsea.sub))) 
              ledge.res[[i]][ledge.dat[[j]],j] <- 1
        }
    }
    
    return(ledge.res)
}

#' Read in a sample data table.
#'
#' Read in a .txt or .csv file containing sample names, group identifiers,
#' replicate identifiers, and any other sample data. Sample names must be in the
#' first column and must correspond with sample names in the count data file(s).
#' 
#' @export
#'
#' @param dat expression data, read in by read_merge_rcc or read.delim
#' @param file.name the path/name of the .txt or .csv file
#' @param idCol the column name of the sample identifiers in the sample table,
#' which should correspond to the column names in the count table 
#' (default NULL: will assume the first column contains the sample identifiers).
#' @param groupCol the column name of the group identifiers.
#' @param replicateCol the column name of the replicate identifiers (default
#' NULL). Multiple replicates of the same sample will have the same value in 
#' this column.
#' 
#' @return The list with the expression data, now combined with the sample
#' information
#' 
#' @examples 
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' sample_info <- system.file("extdata", "GSE117751_sample_data.csv", 
#'                            package = "NanoTube")
#' 
#' dat <- read_merge_rcc(list.files(example_data, full.names = TRUE))
#'
#' # Merge expression data with sample info
#' dat <- read_sampleData(dat, file.name = sample_info,
#'                        groupCol = "Sample_Diagnosis")

read_sampleData <- function(dat, file.name, 
                            idCol = NULL, groupCol, replicateCol = NULL) {
    # Read in the file
    file.extension <- substr(file.name, (nchar(file.name)-3), nchar(file.name))
    if (file.extension %in% c(".txt", ".TXT")) {
        sampDat <- read.delim(file.name, stringsAsFactors = FALSE)
    } else if (file.extension %in% c(".csv", ".CSV")) {
        sampDat <- read.csv(file.name, stringsAsFactors = FALSE)
    } else {
        stop("\nSample data table must be in .csv (comma-delimited) or .txt 
             (tab-delimited) format.\n")
    }
    
    # Set the sample ID's
    if (is.null(idCol)) {
        warning("\nidCol not provided. Assuming the first column of '",
                gsub(".*\\/", "", file.name), "' contains sample ID's.\n")
        
        rownames(sampDat) <- sampDat[,1]
    }
    else {
        if (idCol %in% colnames(sampDat)) {
            rownames(sampDat) <- sampDat[,idCol]
        } else {
            stop("\n'", idCol, "' is not one of the columns in '",
                 gsub(".*\\/", "", file.name), "'.\n")
        }
    }
    
    # Check if expression data and meta data have the same number of samples.
    if (ncol(dat$exprs) == nrow(sampDat)) {
        if (all(rownames(sampDat) %in% colnames(dat$exprs))) {
            # If both files have same sample names, ensure that samples 
            # are in the same order.
            dat$samples <- sampDat[colnames(dat$exprs),]
        } else {
            # If files have different sample names (or no sample names in 
            # meta file), assume that they are in the same order (and warn).
            dat$samples <- sampDat
            rownames(dat$samples) <- colnames(dat$exprs)
            
            warning("\nSample names in the two files don't match. NanoTube is\n", 
                "assuming that samples are in the same order. Please confirm\n", 
                "with your data.\n")
        }
    } else {
        stop("\nConflict: There is expression data from ", ncol(dat$exprs), 
             " samples, but sample data from ", nrow(sampDat), " samples.\n")
    }
    
    # Check that group column and replicates column are present in the table.
    if (!is.null(groupCol)) {
      if (groupCol %in% colnames(dat$samples)) {
        dat$groups <- dat$samples[,groupCol]
      } else {
        stop("\n'", groupCol, "' is not one of the column names in the
             sample table.\n")
      }
    }
    
    
    if (!is.null(replicateCol)) {
        if (replicateCol %in% colnames(dat$samples)) {
            dat$replicates <- dat$samples[,replicateCol]
        } else {
            stop("\n'", replicateCol, "' is not one of the column names in the
                 sample table.\n")
        }
    }
    
    return(dat)

}

#' Cluster gene set analysis results
#'
#' Groups the pathway analysis results (using limmaToFGSEA or nsdiffToFGSEA) 
#' based on the enriched gene sets' leading edges. If the calculated distance 
#' metric is lower than the given threshold (i.e. the gene sets have highly 
#' overlapping leading edge genes), these gene sets will be joined to a single 
#' gene set "cluster." Or if 'ngroups' is specified, gene sets will be clustered
#' by similarity into that number of groups.
#'
#' @export
#'
#' @param gsea.res Results from pathway analysis for a single comparison, 
#' using limmaToFSEA.
#' @param l.edge Leading edge result from fgseaToLEdge.
#' @param join.threshold The threshold distance to join gene sets. Gene sets 
#' with a distance below this value will be joined to a single "cluster."
#' @param ngroups The desired number of gene set groups. Either
#' 'join.threshold' or 'ngroups' must be specified, 'ngroups' takes priority
#' if both are specified.
#' @param dist.method Method for distance calculation (see options for dist()).
#' We recommend the 'binary' (also known as Jaccard) distance.
#' @param returns Either "signif" or "all". This argument defines whether only 
#' significantly enriched gene sets are included in the output table, or if 
#' the full results are included. Regardless of this selection, only 
#' significantly enriched gene sets are clustered.
#' @return A data frame including the FGSEA results, plus two additional 
#' columns for the clustering results:
#' @return \item{Cluster}{The cluster that the gene set was assigned to. Gene 
#' sets in the same cluster have a distance below the join.threshold.}
#' @return \item{best}{Whether the gene set is the most enriched (by p-value) 
#' in a given cluster.}
#'
#' @examples
#' data("ExamplePathways")
#' data("ExampleResults") # Results from runLimmaAnalysis
#'
#' fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways,
#'                              min.set = 5, rank.by = "t")
#'
#' leadingEdge <- fgseaToLEdge(fgseaResults, cutoff.type = "padj", 
#'                             cutoff = 0.25)
#'
#' # Group the results, and only returns those satisfying the cutoff specified 
#' # in leadingEdge()
#' groupedResults <- groupFGSEA(fgseaResults$Autoimmune.retinopathy, 
#'                              leadingEdge$Autoimmune.retinopathy,
#'                              join.threshold = 0.5,
#'                              returns = "signif")

groupFGSEA <- function(gsea.res, l.edge,
                               join.threshold = NULL,
                               ngroups = NULL,
                               dist.method = "binary",
                               returns = c("signif", "all")) {
  
    # Convert fgsea results to be usable for this
    if (colnames(gsea.res)[2] != "p.val") {
        colnames(gsea.res)[c(2,3)] <- c("p.val", "p.adj")
        gsea.res <- as.data.frame(gsea.res[,seq_len(7)])
        rownames(gsea.res) <- gsea.res$pathway
    }
    
    gsea.sub <- gsea.res[rownames(gsea.res) %in% colnames(l.edge),]
    gsea.sub <- gsea.sub[order(gsea.sub$p.adj, gsea.sub$p.val, 
                               decreasing = FALSE),]
    l.edge <- l.edge[,rownames(gsea.sub)]
    
    d <- dist(t(l.edge), method = dist.method)
    groups <- cutree(hclust(d), h = join.threshold, k = ngroups)
    
    gsea.sub$Cluster <- groups
    gsea.sub$Cluster.Max <- ifelse(duplicated(groups), yes = "", no = "x")
    
    # Use "signif" if no returns option specified
    returns <- returns[1]
    
    if (returns == "signif") {
        gsea.return <- cbind(gsea.sub, t(l.edge))
        gsea.return <- gsea.return[order(gsea.return$Cluster, 
                                         decreasing=FALSE),]
    } else {
        gsea.non <- gsea.res[!(rownames(gsea.res) %in% colnames(l.edge)),]
        gsea.non$Cluster <- NA
        gsea.non$Cluster.Max <- ""
        gsea.return <- rbind(gsea.sub, gsea.non)
    }
  
    return(gsea.return)
    
}

#' Make master table of all GSEA results
#'
#' This function clusters GSEA results by leading edge similarity, and then 
#' combines to a data frame or text file.
#'
#' @param genesetResults Results from pathway analysis using limmaToFGSEA.
#' @param leadingEdge Results from fgseaToLEdge
#' @param join.threshold The threshold distance to join gene sets. Gene sets 
#' with a distance below this value will be joined to a single "cluster."
#' @param ngroups The desired number of gene set groups. Either 
#' 'join.threshold' or 'ngroups' must be specified, 'ngroups' takes priority 
#' if both are specified.
#' @param dist.method Method for distance calculation (see options for dist()).
#' We recommend the 'binary' (also known as Jaccard) distance.
#' @param filename File name for the output text file. If NULL (default), data 
#' will be returned as an R data frame.
#' 
#' @return A table of GSEA results, clustered by similarity of leading edge.

makeFGSEAmasterTable <- function(genesetResults, leadingEdge,
                                 join.threshold = 0.5,
                                 ngroups = NULL,
                                 dist.method = "binary",
                                 filename = NULL) {

    geneset.clustered <- list()
  
    for (i in names(genesetResults)) {
      
        if (is.null(leadingEdge[[i]])) {
            temp <-  genesetResults[[i]]
            temp$Cluster <- NA
            temp$Cluster.Max <- ""
            colnames(temp)[2:3] <- c("p.val", "p.adj")
            geneset.clustered[[i]] <- temp[,-8]
        } else {
            geneset.clustered[[i]] <- groupFGSEA(genesetResults[[i]], 
                                                 leadingEdge[[i]],
                                                 join.threshold, ngroups,
                                                 dist.method, returns="all")
        }
        
        colnames(geneset.clustered[[i]])[-1] <- 
          paste0(colnames(geneset.clustered[[i]])[-1], " (", i, ")")
    }
  
    dat.xpt <- Reduce(function(...) merge(..., all=TRUE, by="pathway"), 
                      geneset.clustered)
  
    if (!is.null(filename)) {
        write.table(dat.xpt, filename,
                    row.names = FALSE, col.names = TRUE, 
                    sep = "\t", quote = FALSE)
    } else {
        return(dat.xpt)
    }
}

#' Conduct differential expression analysis
#'
#' Use Limma to conduct a simple differential expression analysis. All groups 
#' are compared against the base.group, and empirical Bayes method is used to
#' identify significantly differentially expressed genes. Alternatively, a
#' design matrix can be supplied, as explained in limma::limmaUsersGuide()
#' 
#' @export
#'
#' @param dat NanoString data ExpressionSet, from processNanostringData
#' @param groups character vector, in same order as the samples in dat. NULL
#' if already included in 'dat'
#' @param base.group the group against which other groups are compared (must
#' be one of the levels in 'groups'). Will use the first group if NULL.
#' @param design a design matrix for Limma analysis (default NULL, will do 
#' analysis based on provided 'group' data)
#' @param codeclass.retain The CodeClasses to retain for Limma analysis. 
#' Generally we're interested in endogenous genes, so we keep "endogenous" 
#' only by default. Others can be included by entering a character vector for 
#' this option (see limmaResults3 example). Alternatively, all targets can be 
#' retained by setting this option to ".".
#' @param ... Optional arguments to be passed to limma::lmFit 
#' @return The fit Limma object
#' 
#' @examples 
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' sample_info <- system.file("extdata", "GSE117751_sample_data.csv", 
#'                            package = "NanoTube")
#' 
#' dat <- processNanostringData(nsFiles = example_data,
#'                              sampleTab = sample_info, 
#'                              groupCol = "Sample_Diagnosis")
#' 
#' # Compare the two diseases against healthy controls ("None")
#' limmaResults <- runLimmaAnalysis(dat, base.group = "None")
#' 
#' 
#' # You can also supply a design matrix
#' # Generate fake batch labels
#' batch <- rep(c(0, 1), times = ncol(dat) / 2)
#' 
#' # Reorder groups ("None" first)
#' group <- factor(dat$groups, levels = c("None", "Autoimmune retinopathy", 
#'                                        "Retinitis pigmentosa"))
#' 
#' # Design matrix including sample group and batch
#' design <- model.matrix(~group + batch)
#' 
#' # Analyze data
#' limmaResults2 <- runLimmaAnalysis(dat, design = design)
#' 
#' # Run Limma analysis including endogenous *and* housekeeping genes.
#' limmaResults3 <- runLimmaAnalysis(dat, design = design,
#'                      codeclass.retain = c("endogenous", "housekeeping"))



runLimmaAnalysis <- function(dat, groups = NULL, base.group = NULL,
                             design = NULL,
                             codeclass.retain = "endogenous", ...) {
  
    # Convert codeclass.retain option to format that can be used by grep.
    codeclass.grep <- paste(codeclass.retain, collapse = "|")
    
    # Retain only desired CodeClass/CodeClasses.
    dat.limma <- dat[grep(codeclass.grep, fData(dat)$CodeClass, 
                          ignore.case = TRUE),]
    rownames(dat.limma) <- fData(dat)$Name[grep(codeclass.grep, 
                                                fData(dat)$CodeClass, 
                                                ignore.case = TRUE)]
    
    # If RUV normalization was used, data are already log-transformed.
    if (dat$normalization[1] != "RUVIII") exprs(dat.limma) <- 
        log2(exprs(dat.limma) + 0.5)
  
    
    # Generate the design matrix for model fitting, if not provided
    if (is.null(design)) {
        # If group info was provided in function command, generate a 
        # phenoData table.
        if (!is.null(groups)) {
            sampData <- data.frame(row.names = colnames(dat.limma),
                                   groups = groups)
            
            phenoData(dat.limma) <- AnnotatedDataFrame(sampData)
        }
        
        # If group info was loaded from a sample table, it should be included 
        # in 'dat'.
        if (is.null(groups) & !("groups" %in% colnames(pData(dat)))) {
            stop("Groups not defined. Group identifiers must be loaded during
                 processNanostringData() or runLimmaAnalysis().")
        }
        
        # If base group was not defined, set it as the first group provided.
        if (is.null(base.group)) {
            base.group <- dat.limma$groups[1]
        } else {
            # Check validity of base.group
            if (!(base.group %in% dat.limma$groups)) {
                stop("'base.group' must be in 'groups'")
            }
        }
        
        if (is(dat.limma$groups, "factor")) {
            dat.limma$groups <- 
              factor(dat.limma$groups, 
                     levels = c(base.group, levels(dat.limma$groups)
                                [levels(dat.limma$groups) != base.group]))
        } else {
            dat.limma$groups <- 
              factor(dat.limma$groups, 
                     levels = c(base.group, unique(dat.limma$groups)
                                [unique(dat.limma$groups) != base.group]))
        }
        
        design <- model.matrix(~1 + dat.limma$groups)
        colnames(design) <- 
          gsub(" ", ".", gsub("dat.limma\\$groups", "", colnames(design)))
        colnames(design)[1] <- "Intercept"
      
    } else {
        # Check that design matrix is correct length
        if (nrow(design) != ncol(dat.limma)) {
            stop("Design matrix and expression data
                 do not have the same number of samples.")
        }
    }
    
    fit <- limma::lmFit(dat.limma, design=design, ...)
    limmaFit <- limma::eBayes(fit)
    limmaFit$q.value <- limmaFit$p.value
    for (i in seq_len(ncol(limmaFit$q.value))) limmaFit$q.value[,i] <- 
        p.adjust(limmaFit$p.value[,i], method = "BH")
  
    limmaFit$eset <- dat.limma
    
    return(limmaFit)
}

#' Postprocessing for GSEA analyses
#'
#' Clusters GSEA results by leading edge genes, and writes reports showing
#' gene expression profiles of these genes.
#' 
#' @export
#'
#' @param genesetResults Results from pathway analysis using limmaToFGSEA.
#' @param leadingEdge Results from fgseaToLEdge
#' @param limmaResults Results from runLimmaAnalysis
#' @param join.threshold The threshold distance to join gene sets. Gene sets 
#' with a distance below this value will be joined to a single "cluster."
#' @param ngroups The desired number of gene set groups. Either 
#' 'join.threshold' or 'ngroups' must be specified, 'ngroups' takes priority 
#' if both are specified.
#' @param dist.method Method for distance calculation (see options for dist()).
#' We recommend the 'binary' (also known as Jaccard) distance.
#' @param reportDir Directory for the GSEA reports (each comparison will be
#' a separate txt file). Directory will be created if it does not exist.
#' 
#' @return A table of gene set analysis results, as well as reports showing
#' differential expression of leading edge genes.
#' 
#' @examples 
#' data("ExamplePathways")
#' data("ExampleResults") # Results from runLimmaAnalysis
#' 
#' fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways)
#' 
#' leadingEdge <- fgseaToLEdge(fgseaResults, cutoff.type = "padj", cutoff = 0.1)
#' 
#' \donttest{
#' fgseaPostprocessing(fgseaResults, leadingEdge, 
#'                     limmaResults = ExampleResults,
#'                     join.threshold = 0.5,
#'                     reportDir = "GSEAresults")
#' }


fgseaPostprocessing <- function(genesetResults, leadingEdge, limmaResults,
                               join.threshold = 0.5, ngroups = NULL,
                               dist.method = "binary", reportDir) {

    dir.create(reportDir, showWarnings = FALSE)
  
    makeFGSEAmasterTable(genesetResults, leadingEdge,
                         join.threshold, 
                         filename = paste0(reportDir, "/gsea_summary.txt"))
  
    for (i in names(genesetResults)) {
        if(!is.null(leadingEdge[[i]])){
            grouped <-   groupFGSEA(genesetResults[[i]],
                                       leadingEdge[[i]],
                                       join.threshold,
                                       ngroups, dist.method,
                                       returns = "signif")
            tab <- groupedGSEAtoStackedReport(grouped, 
                                              leadingEdge[[i]],
                                              limmaResults)
          
            write.table(tab, file = paste0(reportDir, "/", i, ".txt"),
                        sep = "\t", quote = FALSE, 
                        row.names = FALSE, col.names = TRUE)
        }
    }
  
}

#' Process NanoString nCounter gene expression data.
#'
#' This function reads in a zip file or folder containing multiple .rcc files 
#' (or a txt/csv file containing raw count data), and then optionally conducts 
#' positive control normalization, background correction, and housekeeping 
#' normalization.
#' 
#' @export
#'
#' @param nsFiles file path (or zip file) containing the .rcc files, or multiple
#' directories in a character vector, or a single text/csv file containing the 
#' combined counts, or .rcc files in a character vector.
#' @param sampleTab .txt (tab-delimited) or .csv (comma-delimited) file 
#' containing sample data table (optional, default NULL)
#' @param idCol the column name of the sample identifiers in the sample table,
#' which should correspond to the column names in the count table 
#' (default NULL: will assume the first column contains the sample identifiers)
#' @param groupCol the column name of the group identifiers in the sample table.
#' @param replicateCol the column name of the technical replicate identifiers 
#' (default NULL). Multiple replicates of the same sample will have the same 
#' value in this column. Replicates are used to improve normalization 
#' performance in the "RUVIII" method; otherwise they are averaged.
#' @param normalization If "nSolver" (default), continues with background, 
#' positive control, and housekeeping control normalization steps to return
#' a NanoStringSet of normalized data. If "RUVIII", runs RUV normalization using 
#' controls, housekeeping genes and technical replicates. If "RUVg", runs RUV 
#' normalization using housekeeping genes. If "none", returns a 
#' NanoStringSet with the raw counts, suitable for running NanoStringDiff.
#' @param bgType (Only if normalization is not "none") Type of background 
#' correction to use: "threshold" sets a threshold for N standard deviations 
#' above the mean of negative controls. "t.test" conducts a one-sided t test 
#' for each gene against all negative controls. "none" to skip background removal
#' @param bgThreshold If bgType=="threshold", number of sd's above the mean to 
#' set as threshold for background correction.
#' @param bgProportion If bgType=="threshold", proportion of samples that a gene
#' must be above threshold to be included in analysis.
#' @param bgPVal If bgType=="t.test", p-value threshold to use for gene to be 
#' included in analysis.
#' @param bgSubtract Should calculated background levels be subtracted from 
#' reported expressions? If TRUE, will subtract mean+numSD*sd of the negative 
#' controls from the endogenous genes, and then set negative values to zero 
#' (default FALSE)
#' @param n.unwanted The number of unwanted factors to use (for RUVIII or RUVg
#' normalization only). If NULL (default), the maximum possible value will
#' be identified and used. 
#' @param RUVg.drop The number of singular values to drop for RUVg normalization
#' (see RUVSeq::RUVg)
#' @param housekeeping vector of genes (symbols or accession) to use for 
#' housekeeping correction ("nCounter" or "RUVg" normalization). 
#' If NULL, will use genes listed as "Housekeeping" under CodeClass.
#' @param skip.housekeeping Skip housekeeping normalization? (default FALSE)
#' @param includeQC Should we include the QC from the .rcc files? This can 
#' cause errors, particularly when reading in files from multiple experiments.
#' @param sampIds a vector of sample identifiers, important if there are 
#' technical replicates. Currently, this function averages technical replicates.
#' sampIds will be extracted from the replicateCol in the sampleTab, if 
#' provided.
#' @param output.format If "list", will return the normalized (optional) and raw
#' expression data, as well as various QC and relevant information tables. If 
#' "ExpressionSet" (default), will convert to an n*p ExpressionSet, with n rows
#' representing genes and p columns representing samples. ExpressionSet objects
#' are required for some steps, such as runLimmaAnalysis.
#' @param logfile a filename for the logfile (optional). If blank, will print 
#' warnings to screen.
#'
#' @return An list or ExpressionSet containing the raw and/or normalized 
#' counts, dictionary, and sample info if provided
#' 
#' @examples 
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' sample_data <- system.file("extdata", "GSE117751_sample_data.csv", 
#'                            package = "NanoTube")
#' 
#' # Process NanoString data from RCC files present in example_data folder.
#' # Use standard nCounter normalization, removing genes that do
#' # pass a t test against negative control genes with p < 0.05. Return the
#' # result as an "ExpressionSet".
#' 
#' dat <- processNanostringData(nsFiles = example_data,
#'                              sampleTab = sample_data, 
#'                              groupCol = "Sample_Diagnosis",
#'                              normalization = "nSolver",
#'                              bgType = "t.test", bgPVal = 0.01,
#'                              output.format = "ExpressionSet")
#' 
#' # Load NanoString data from a csv file (from NanoString's RCC Collector tool,
#' # for example). Skip normalization by setting 'normalization = "none"'.
#' 
#' csv_data <- system.file("extdata", "GSE117751_expression_matrix.csv", 
#'                         package = "NanoTube")
#' dat <- processNanostringData(nsFile = csv_data,
#'                               sampleTab = sample_data, 
#'                               idCol = "GEO_Accession", 
#'                               groupCol = "Sample_Diagnosis",
#'                               normalization = "none")
#'                               
#' # Load NanoString data from RCC files, using a threshold background level for
#' # removing low-expressed genes. Also, specify which genes to use for 
#' # housekeeping normalization. Save the result in "list" format (useful for
#' # some QC functions) instead of an "ExpressionSet".
#' 
#' dat <- processNanostringData(nsFiles = example_data,
#'                              sampleTab = sample_data, 
#'                              groupCol = "Sample_Diagnosis",
#'                              normalization = "nSolver",
#'                              bgType = "threshold", 
#'                              bgThreshold = 2, bgProportion = 0.5,
#'                              housekeeping = c("TUBB", "TBP", "POLR2A", 
#'                                               "GUSB", "SDHA"),
#'                              output.format = "list")

processNanostringData <- function(nsFiles,
                                  sampleTab = NULL, 
                                  idCol = NULL, groupCol = NULL, 
                                  replicateCol = NULL,
                                  normalization = c("nSolver", "RUVIII", "RUVg", "none"),
                                  bgType = c("threshold", "t.test", "none"),
                                  bgThreshold = 2, bgProportion = 0.5, 
                                  bgPVal = 0.001, bgSubtract = FALSE,
                                  n.unwanted = NULL, RUVg.drop = 0,
                                  housekeeping = NULL, 
                                  skip.housekeeping = FALSE,
                                  includeQC = FALSE,
                                  sampIds = NULL,
                                  output.format = c("ExpressionSet", "list"),
                                  logfile = "") {

    # Quick error checking:
    if (bgThreshold < 0) {
        warning(cat("\nNegative background threshold detected. This is the 
                    number of \nstandard deviations above the background mean, 
                    and should be \n0 or positive. Setting to 0...\n", 
                    file=logfile, append=TRUE))
        bgThreshold <- 0
    }
    if (bgProportion < 0 | bgProportion > 1) {
        stop(cat("\nProportion should be between 0 and 1, the proportion of 
                 samples \nthat must have greater expression than background to
                 keep for \nanalysis. Stopping...\n", 
                 file=logfile, append=TRUE))
    }
  
    file.extension <- substr(nsFiles[1], 
                             (nchar(nsFiles[1])-3), nchar(nsFiles[1]))
    
    # Read in expression data from individual rcc files, 
    # or merged txt or csv files.
    
    if (file.extension %in% c(".txt", ".TXT", ".csv", ".CSV")) {
      
        # Read in merged count data
        cat("\nLoading count data......",
            file=logfile, append=TRUE)
        
        if (file.extension %in% c(".txt", ".TXT")) {
            tabData <- read.delim(nsFiles,
                                stringsAsFactors = FALSE)
        } else {
            tabData <- read.csv(nsFiles,
                                stringsAsFactors = FALSE)
        }
        
        dat <- list(exprs = tabData[,-seq_len(3)],
                    dict = tabData[,seq_len(3)])
        rownames(dat$exprs) <- rownames(dat$dict) <- tabData$Accession
        
        # Remove periods or spaces from dictionary colnames
        colnames(dat$dict) <- gsub("\\.| ", "", colnames(dat$dict))
      
    } else if (file.extension %in% c(".RCC", ".rcc")) {
      
        fileNames <- nsFiles[ grep("\\.rcc$", tolower(nsFiles)) ]
        cat("\nReading in .RCC files......", file = logfile, 
            append = TRUE)
        dat <- read_merge_rcc(fileNames, includeQC, logfile)
      
    } else {
      
        # Extract from nsFiles, if zipped
        if (file.extension %in% c(".zip", ".ZIP")){
            nsFiles <- unzip_dirs(nsFiles)
        } else if (file.extension %in% c(".tar", ".TAR")){
          nsFiles <- untar_dirs(nsFiles)
        }
        
        # Get filenames (combines files from multiple directories if necessary)
        fileNames <- unlist(lapply(nsFiles, list.files, recursive = TRUE, 
                                   full.names = TRUE))
        
        # Retain only fileNames ending in "RCC" or ".gz"
        # ('tolower' function makes it case-insensitive)
        fileNames <- fileNames[
          tolower(substr(fileNames, 
                         start = nchar(fileNames)-2, 
                         stop = nchar(fileNames))) %in% c("rcc", ".gz")]
        
        cat("\nReading in .RCC files......", file=logfile, append=TRUE)
        
        # Read in RCC files
        dat <- read_merge_rcc(fileNames, includeQC, logfile)
    }
    
    # Read in sample data file, if provided.
    if (!is.null(sampleTab)) dat <- read_sampleData(dat, file.name = sampleTab,
                                      idCol = idCol, groupCol = groupCol, 
                                      replicateCol = replicateCol)
    
    # Mark specified genes as housekeeping (may already be marked)
    dat$dict$CodeClass[dat$dict$Name %in% housekeeping | 
                         dat$dict$Accession %in% housekeeping] <- "Housekeeping"
    
    # Average counts for technical replicates or normalize using RUV.
    # Use the replicate ID's extracted from the sampleTab, if applicable.
    if ("replicates" %in% names(dat)) sampIds <- dat$replicates
    
    # Or use the ID's provided directly.
    # Then average replicates, or normalize using "RUV".
    normalization <- normalization[1]
    bgType <- bgType[1]
    output.format <- output.format[1]
    
    if ((!is.null(sampIds) & any(duplicated(sampIds))) |
        normalization != "RUVIII") {
      if (is.null(sampIds)) sampIds <- seq_len(ncol(dat$exprs))
        
        dupSamps <- names(table(sampIds)[table(sampIds) > 1])
        
        if (normalization == "RUVIII") {
            cat("\nConducting RUVIII normalization......",
                file=logfile, append=TRUE)
            
            # Save a copy of the raw expression data
            dat$exprs.raw <- dat$exprs
            dat$samples.raw <- dat$samples
            dat$dict.raw <- dat$dict
            
            if (bgType != "none") {
              dat <- remove_background(dat, mode = bgType, 
                                            numSD = bgThreshold, 
                                            proportionReq = bgProportion,
                                            pval = bgPVal, subtract = bgSubtract)
            }
            
            # Normalize using RUV: Use all genes as control genes to start 
            # (recommended by RUV authors)
            dat$exprs <- t(ruv::RUVIII(t(log2(dat$exprs+0.5)), M = sampIds,
                                       ctl = seq_len(nrow(dat$exprs)), 
                                       k = n.unwanted))
          
        } else {
            cat("\nAveraging technical replicates.....",
                file=logfile, append=TRUE)
            for (i in dupSamps) {
                dat$exprs[,sampIds == i] <- rowMeans(dat$exprs[,sampIds == i])
            }
        }
        
        # Remove duplicates
        dat$exprs <- dat$exprs[,!duplicated(sampIds)]
        dat$samples <- dat$samples[!duplicated(sampIds),]
        if (includeQC) dat$qc <- dat$qc[!duplicated(sampIds),]
        if ("groups" %in% names(dat))
            dat$groups <- dat$groups[!duplicated(sampIds)]
    }
    
    # Normalize using nSolver recommended method:
    if (normalization == "nSolver") {
        cat("\nCalculating positive scale factors......",
            file=logfile, append=TRUE)
        dat.norm <- normalize_pos_controls(dat, logfile)
        
        # Remove genes that fail background check
        cat("\nChecking endogenous genes against background threshold......",
            file=logfile, append=TRUE)
    
        if (bgType != "none") {
          dat.norm <- remove_background(dat.norm, mode = bgType, 
                                      numSD = bgThreshold, 
                                      proportionReq = bgProportion,
                                      pval = bgPVal, subtract = bgSubtract)
        }
        
        if (!skip.housekeeping) {
            cat("\nConducting housekeeping normalization......",
                file=logfile, append=TRUE)
            dat.norm <- normalize_housekeeping(dat.norm, housekeeping)
        }
        
        # Save a copy of the raw data (only used if output.format == "list")
        dat.out <- dat.norm
        dat.out$exprs.raw <- dat$exprs
        dat.out$dict.raw <- dat$dict
        dat <- dat.out
        
    } else if (normalization == "RUVg") {
      
      cat("\nConducting RUVg normalization......",
          file=logfile, append=TRUE)
      
      dat$exprs.raw <- dat$exprs
      dat$dict.raw <- dat$dict
      
      if (bgType != "none") {
        dat <- remove_background(dat, mode = bgType, 
                                      numSD = bgThreshold, 
                                      proportionReq = bgProportion,
                                      pval = bgPVal, subtract = bgSubtract)
      }
      
      if (is.null(housekeeping)) {
        housekeeping <- rownames(dat$dict)[
          grep("housekeeping", dat$dict$CodeClass, ignore.case = TRUE)]
      } else if (all(housekeeping %in% dat$dict$Accession)) {
        housekeeping <- rownames(dat$dict)[dat$dict$Accession %in% housekeeping]
      } else if (all(housekeeping %in% dat$dict$Name)) {
        housekeeping <- rownames(dat$dict)[dat$dict$Name %in% housekeeping]
      } else {
        stop(cat("\nHousekeeping gene(s) not found in dataset:", 
                 housekeeping[!(housekeeping %in% dat$dict$Name | 
                                  housekeeping %in% dat$dict$Accession)],
                 "\n", file=logfile, append=TRUE))
      }

      if (is.null(n.unwanted)) n.unwanted <- 1
    
      dat$exprs <- RUVSeq::RUVg(dat$exprs, housekeeping, 
                                k = n.unwanted, drop = RUVg.drop)$normalizedCounts
      
    }
    
    if (output.format == "list") {
        dat$normalization <- normalization
        
        if (normalization == "none") {
            dat$exprs.raw <- dat$exprs
            dat$dict.raw <- dat$dict
        }
        
        return(dat)
      
    } else {
        dat.out <- ExpressionSet(assayData = as.matrix(dat$exprs),
                                 featureData = AnnotatedDataFrame(dat$dict))
        
        if ("samples" %in% names(dat))
            phenoData(dat.out) <- AnnotatedDataFrame(dat$samples)
        if ("groups" %in% names(dat)) phenoData(dat.out)$groups <- dat$groups
        
        phenoData(dat.out)$normalization <- normalization
        
        if (includeQC)
          phenoData(dat.out) <-
              AnnotatedDataFrame(cbind(pData(dat.out), dat$qc))
        
        return(dat.out)
    }
}

#' Unzip
#'
#' Unzips provided list of directories
#' 
#' @export
#'
#' @param fileDirs character list of zip files
#' @return Names of now-unzipped directories

unzip_dirs <- function(fileDirs) {
    print(getwd())
    new.fileDirs <- gsub("\\.zip|\\.ZIP", "", fileDirs)
    for (i in seq_along(fileDirs)) {
        unzip(fileDirs[i], exdir = new.fileDirs[i])
    }
    return(new.fileDirs)
}

#' Untar
#'
#' Untars provided list of directories (analogous to unzip_dirs)
#' 
#' @export
#'
#' @param fileDirs character list of tar files
#' @return Names of now-untarred directories

untar_dirs <- function(fileDirs) {
    print(getwd())
    new.fileDirs <- gsub("\\.tar|\\.TAR", "", fileDirs)
    for (i in seq_along(fileDirs)) {
        untar(fileDirs[i], exdir = new.fileDirs[i])
    }
    return(new.fileDirs)
}

## ----setup, echo = FALSE, results = "hide"------------------------------------
knitr::opts_chunk$set(tidy = FALSE,
                      cache = FALSE,
                      dev = "png",
                      message = FALSE, error = FALSE, warning = TRUE,
                      fig.height = 4, fig.width = 4)

library(kableExtra)

## ----Basic1-------------------------------------------------------------------

library(NanoTube)

example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
sample_info <- system.file("extdata", 
                           "GSE117751_sample_data.csv", 
                           package = "NanoTube")


## ----Basic2-------------------------------------------------------------------

dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info,
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis")


## ----BasicGroups--------------------------------------------------------------

table(dat$groups)


## ----Basic3-------------------------------------------------------------------

limmaResults <- runLimmaAnalysis(dat, base.group = "None")


## ----Basic3b------------------------------------------------------------------

limmaStats <- makeDiffExprFile(limmaResults, filename = NULL, returns = "stats")
limmaStats <- as.data.frame(limmaStats)

# Rounding for clarity
limmaTab <- head(limmaStats[order(limmaStats$`p-val (Autoimmune.retinopathy)`, 
                      decreasing = FALSE), 1:4])
limmaTab[,1] <- format(limmaTab[,1], digits = 2, nsmall = 1)
limmaTab[,3] <- format(limmaTab[,3], digits = 1, scientific = TRUE)
limmaTab[,4] <- format(limmaTab[,4], digits = 1, nsmall = 1)

# Order by lowest to highest p-value for 'Autoimmune Retinopathy' vs. 'None'
knitr::kable(head(limmaTab), 
      row.names = TRUE, format = "html", align = "c")



## ----BasicVol-----------------------------------------------------------------

deVolcano(limmaResults, plotContrast = "Autoimmune.retinopathy")


## ----Basic4-------------------------------------------------------------------

data("ExamplePathways")

fgseaResults <- limmaToFGSEA(limmaResults, gene.sets = ExamplePathways)

# FGSEA was run separately for Autoimmune Retinopathy vs. None and 
# Retinitis Pigmentosa vs. None
names(fgseaResults)


## ----BasicTab-----------------------------------------------------------------

fgseaTab <- head(fgseaResults$Autoimmune.retinopathy[
             order(fgseaResults$Autoimmune.retinopathy$pval, 
                   decreasing = FALSE),])
fgseaTab[,2:6] <- lapply(fgseaTab[,2:6], format, digits = 1, nsmall = 1)

knitr::kable(fgseaTab, 
      row.names = FALSE, format = "html", align = "c")



## ----proc1--------------------------------------------------------------------

# Without supplying idCol, a warning is returned.
# The CSV file contains samples in the proper order, so this is technically ok.
dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver",
                             bgType = "t.test", bgPVal = 0.01,
                             skip.housekeeping = FALSE,
                             output.format = "ExpressionSet")


## ----proc1idCol---------------------------------------------------------------

# With idCol
dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver",
                             bgType = "t.test", bgPVal = 0.01,
                             skip.housekeeping = FALSE,
                             output.format = "ExpressionSet")


## ----proc2--------------------------------------------------------------------

csv_data <- system.file("extdata", 
                        "GSE117751_expression_matrix.csv", 
                        package = "NanoTube")

dat2 <- processNanostringData(nsFile = csv_data,
                             sampleTab = sample_info, 
                             idCol = "GEO_Accession", 
                             groupCol = "Sample_Diagnosis",
                             normalization = "none")


## ----norm1--------------------------------------------------------------------

# Set housekeeping genes manually (optional)
hk.genes <- c("TUBB", "TBP", "POLR2A", "GUSB", "SDHA")

dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver",
                             bgType = "t.test", bgPVal = 0.01,
                             housekeeping = hk.genes, skip.housekeeping = FALSE)


## ----norm2--------------------------------------------------------------------

# Automatically detect housekeeping genes

dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver",
                             bgType = "t.test", bgPVal = 0.01)


## ----normRUVIII---------------------------------------------------------------

# This sample data table contains fake replicate identifiers (for use in example).
sample_info_reps <- system.file("extdata", 
                           "GSE117751_sample_data_replicates.csv", 
                           package = "NanoTube")

datRUVIII <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info_reps, 
                             idCol = "RCC_Name",
                             replicateCol = "Replicate_ID",
                             groupCol = "Sample_Diagnosis",
                             normalization = "RUVIII",
                             bgType = "t.test", bgPVal = 0.01,
                             n.unwanted = 1)


## ----normRUVg-----------------------------------------------------------------

hk.genes <- c("TUBB", "TBP", "POLR2A", "GUSB", "SDHA")

datRUVg <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "RUVg",
                             bgType = "t.test", bgPVal = 0.01,
                             n.unwanted = 1, RUVg.drop = 0,
                             housekeeping = hk.genes)


## ----normRUVg2----------------------------------------------------------------

# Housekeeping genes are automatically identified
datRUVg <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "RUVg",
                             bgType = "t.test", bgPVal = 0.01,
                             n.unwanted = 1, RUVg.drop = 0)


## ----rle, fig.width = 6-------------------------------------------------------

# Here's how to do it with the NanoString ExpressionSet. If your data set is in
# list form, you would use dat$exprs instead of exprs(dat).

ruv::ruv_rle(t(log2(exprs(dat))), ylim = c(-2, 2))


## ----pca, fig.width = 6-------------------------------------------------------

dat <- processNanostringData(example_data, 
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver", 
                             bgType = "t.test", 
                             bgPVal = 0.01)

library(plotly) 
nanostringPCA(dat, interactive.plot = TRUE)$plt


## ----pca2, fig.width = 6------------------------------------------------------

nanostringPCA(dat, pc1=3, pc2=4, interactive.plot = FALSE)$plt


## ----qc1----------------------------------------------------------------------

dat <- processNanostringData(example_data, 
                             idCol = "RCC_Name",
                             output.format = "list",
                             includeQC = TRUE)


## ----qc2----------------------------------------------------------------------

head(dat$qc)[,1:5]


## ----qcPos1-------------------------------------------------------------------

posQC <- positiveQC(dat)

knitr::kable(head(posQC$tab), 
      row.names = FALSE, format = "html", align = "c", digits = 2)


## ----qcPos2, fig.width = 7----------------------------------------------------

posQC2 <- positiveQC(dat, samples = 1:6)

posQC2$plt


## ----qcNeg1-------------------------------------------------------------------

negQC <- negativeQC(dat, interactive.plot = FALSE)

knitr::kable(head(negQC$tab), 
      row.names = TRUE, format = "html", align = "c")


## ----qcNeg2, fig.height = 7, fig.width = 7------------------------------------

negQC$plt


## ----qcHK1--------------------------------------------------------------------
signif(head(dat$hk.scalefactors), digits = 2)

## ----gps----------------------------------------------------------------------

dat <- processNanostringData(example_data, 
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver", 
                             bgType = "t.test", 
                             bgPVal = 0.01)

table(dat$groups)


## ----deLimma------------------------------------------------------------------

limmaResults <- runLimmaAnalysis(dat, base.group = "None")

head(signif(limmaResults$coefficients, digits = 2))


## ----deLimma2, fig.height = 3.5-----------------------------------------------

# Generate fake batch labels
batch <- rep(c(0, 1), times = ncol(dat) / 2)

# Reorder groups ("None" first)
group <- factor(dat$groups, 
                levels = c("None", "Autoimmune retinopathy", 
                           "Retinitis pigmentosa"))

# Design matrix including sample group and batch
design <- model.matrix(~group + batch)

# Analyze data
limmaResults2 <- runLimmaAnalysis(dat, design = design)

# We can see there is no batch effect in this data, due to the somewhat uniform
# distribution of p-values. This is good, since the batches are fake.
hist(limmaResults2$p.value[,"batch"], main = "p-values of Batch terms")


## ----desInput-----------------------------------------------------------------

sample_info_design <- system.file("extdata", 
                           "GSE117751_design_matrix.csv", 
                           package = "NanoTube")

datDes <- processNanostringData(example_data, 
                             sampleTab = sample_info_design, 
                             idCol = "RCC_Name",
                             normalization = "nSolver", 
                             bgType = "t.test", 
                             bgPVal = 0.01)

head(pData(datDes))


## ----limma3-------------------------------------------------------------------

# Analyze data
limmaResults3 <- runLimmaAnalysis(datDes, 
                                  design = pData(datDes)[,2:(ncol(pData(datDes))-1)])


hist(limmaResults3$p.value[,"Age"], main = "p-values for Age term")


## ----volcano------------------------------------------------------------------

deVolcano(limmaResults, y.var = "p.value")


## ----volcano2-----------------------------------------------------------------

deVolcano(limmaResults, plotContrast = "Autoimmune.retinopathy", 
          y.var = "p.value") +
  geom_hline(yintercept = 2, linetype = "dashed", colour = "darkred") +
  geom_vline(xintercept = 0.5, linetype = "dashed", colour = "darkred") +
  geom_vline(xintercept = -0.5, linetype = "dashed", colour = "darkred")



## ----deExport-----------------------------------------------------------------

limmaStats <- makeDiffExprFile(limmaResults, filename = NULL, returns = "stats")
limmaStats <- as.data.frame(limmaStats)

# Order by lowest to highest p-value for 'Autoimmune Retinopathy' vs. 'None'
knitr::kable(head(limmaStats[order(limmaStats$`p-val (Autoimmune.retinopathy)`, 
                                   decreasing = FALSE),]), 
      row.names = TRUE, format = "html", align = "c")



## ----nsDiffConvert------------------------------------------------------------

# Remember to set normalization = "none"
datNoNorm <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "none")

nsDiffSet <- makeNanoStringSetFromEset(datNoNorm)

colnames(pData(nsDiffSet))


## ----nsDiffRun, eval = FALSE--------------------------------------------------
# 
# ### This block is not run! ###
# 
# # This step is fast
# nsDiffSet <- NanoStringDiff::estNormalizationFactors(nsDiffSet)
# 
# # This step likely to take multiple hours on standard desktop computers.
# result <- NanoStringDiff::glm.LRT(nsDiffSet,
#                                   design.full = as.matrix(pData(nsDiffSet)),
#                                   contrast = c(1, -1, 0))
#                                   #Contrast: Autoimmune retinopathy vs. None
# 

## ----gsea1--------------------------------------------------------------------
data("ExamplePathways")

fgseaResults <- limmaToFGSEA(limmaResults, gene.sets = ExamplePathways, 
                            min.set = 5, rank.by = "t",
                            skip.first = TRUE)


names(fgseaResults)


## ----gsea2--------------------------------------------------------------------

fgseaTab <- head(fgseaResults$Autoimmune.retinopathy[
             order(fgseaResults$Autoimmune.retinopathy$pval, 
                   decreasing = FALSE),])
fgseaTab[,2:6] <- lapply(fgseaTab[,2:6], format, digits = 1, nsmall = 1)

knitr::kable(fgseaTab, 
      row.names = FALSE, format = "html", align = "c")



## ----gsea3--------------------------------------------------------------------

# Leading edge for pathways with adjusted p < 0.2
leading.edge <- fgseaToLEdge(fgsea.res = fgseaResults,
                             cutoff.type = "padj", cutoff = 0.2) 


## ----gsea4--------------------------------------------------------------------

# Leading edge for pathways with abs(NES) > 1
leading.edge.nes <- fgseaToLEdge(fgsea.res = fgseaResults,
                                cutoff.type = "NES", cutoff = 1,
                                nes.abs.cutoff = TRUE) 

# Leading edge for pathways with NES > 1.5
leading.edge.nes <- fgseaToLEdge(fgsea.res = fgseaResults,
                                cutoff.type = "NES", cutoff = 1.5,
                                nes.abs.cutoff = FALSE) 

# Leading edge for pathways with NES < -0.5
leading.edge.nes <- fgseaToLEdge(fgsea.res = fgseaResults,
                                cutoff.type = "NES", cutoff = -0.5,
                                nes.abs.cutoff = FALSE) 



## ----gsea5, fig.width = 8, fig.height = 3-------------------------------------

pheatmap::pheatmap(t(leading.edge$Autoimmune.retinopathy),
                   legend = FALSE,
                   color = c("white", "black"))


## ----gsea6--------------------------------------------------------------------

# Group pathways with a binary distance below 0.5
fgsea.grouped <- groupFGSEA(fgseaResults$Autoimmune.retinopathy, 
                                   leading.edge$Autoimmune.retinopathy,
                                   join.threshold = 0.5,
                                   dist.method = "binary")

fgseaTab <- head(fgsea.grouped)
fgseaTab[,2:6] <- lapply(fgseaTab[,2:6], format, digits = 1, nsmall = 1)

knitr::kable(fgseaTab, 
      row.names = FALSE, format = "html", align = "c")


## ----exportGSEA---------------------------------------------------------------

fgseaPostprocessingXLSX(genesetResults = fgseaResults,
                        leadingEdge = leading.edge,
                        limmaResults = limmaResults,
                        join.threshold = 0.5,
                        filename = "analysis.xlsx")


## ----exportGSEA2--------------------------------------------------------------

results.AR <- groupedGSEAtoStackedReport( 
    fgsea.grouped,
    leadingEdge = leading.edge$Autoimmune.retinopathy,
    de.fit = limmaResults)

# View Cluster 1 gene statistics
resultsTab <- results.AR[results.AR$Cluster == 1, 1:6]
resultsTab[,2:6] <- lapply(resultsTab[,2:6], format, digits = 1, nsmall = 1)

knitr::kable(resultsTab, 
      row.names = FALSE, format = "html", align = "c")

#' Calculate the geometric mean
#'
#' Calculates the geometric mean of a numeric vector
#'
#' @export
#' 
#' @param x A numeric vector
#' @param na.rm Logical (default TRUE). Should NA values be ignored in this 
#' calculation? If FALSE, a vector containing NA values will return a 
#' geometric mean of NA.
#' @return The geometric mean
#' 
#' @examples
#' gm_mean(c(1, 3, 5))

gm_mean = function(x, na.rm=TRUE){
    exp(sum(log(x[x > 0]), na.rm=na.rm) / length(x))
}

#' Positive control gene normalization
#'
#' Scale genes by the geometric mean of positive control genes. This step is
#' conducted within processNanostringData, when normalization is set to 
#' "nCounter".
#' 
#' @export
#'
#' @param dat NanoString data, including expression matrix and gene dictionary.
#' @param logfile Optional name of logfile to print messages, warnings or 
#' errors.
#' 
#' @return NanoString data, with expression matrix now normalized by 
#' positive control gene expression.
#' 
#' @examples
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' 
#' dat <- read_merge_rcc(list.files(example_data, full.names = TRUE))
#' 
#' # Positive controls are identified in the RCC files, and used to 
#' # normalize the data
#' dat <- normalize_pos_controls(dat)

normalize_pos_controls <- function(dat, logfile="") {

    # Check for positive controls
    if (sum(dat$dict$CodeClass == "Positive") == 0) {
        stop("No positive control genes found (should have CodeClass of 
             'Positive'). Stopping...\n",
             file=logfile)
    }
  
    exprs.dat <- dat$exprs
  
    laneGM <- apply(exprs.dat[dat$dict$CodeClass == "Positive",], 2, gm_mean)
    scale.factor <- laneGM / mean(laneGM)
    
    if (any(scale.factor < 0.3 | scale.factor > 3)) {
        warning(cat("Identified positive scale factor outside recommended range 
                    (0.3-3). \nCheck samples prior to conducting analysis.\n", 
                    file=logfile, append=TRUE))
    }
  
    exprs.dat <- sweep(exprs.dat, 2, scale.factor, '/')
    dat$exprs <- exprs.dat
    dat$pc.scalefactors <- scale.factor
    return(dat)
}

#' Read .rcc file
#'
#' This function reads in a single .rcc file and splits into expression, 
#' sample data, and qc components.
#' 
#' @export
#'
#' @param file file name
#' 
#' @return list containing expression data, sample attributes, and basic qc from
#' the .rcc file.
#' 
#' @examples
#' example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
#' 
#' # First file only
#' single_file <- list.files(example_data, full.names = TRUE)[1]
#' single_dat <- read_rcc(single_file)

read_rcc <- function(file) {

    if (substr(file, start = nchar(file)-2, stop = nchar(file)) == ".gz") {
        file <- gzfile(file)
    }
    
    dat.csv <- read.csv(file,
                        row.names = NULL, col.names = seq_len(4), 
                        header = FALSE, stringsAsFactors = FALSE)
    dat <- list(exprs = dat.csv[(which(dat.csv[,1] == "<Code_Summary>")+1):
                                  (which(dat.csv[,1] == "</Code_Summary>")-1),],
                sample = dat.csv[
                    (which(dat.csv[,1] == "<Sample_Attributes>")+1):
                        (which(dat.csv[,1] == "</Sample_Attributes>")-1), 
                    c(1,2)],
                qc = dat.csv[(which(dat.csv[,1] == "<Lane_Attributes>")+1):
                               (which(dat.csv[,1] == "</Lane_Attributes>")-1), 
                             c(1,2)])
    for (i in seq_len(3)) {
        colnames(dat[[i]]) <- dat[[i]][1,]
        dat[[i]] <- dat[[i]][-1,]
        rownames(dat[[i]]) <- NULL
    }
    dat[[1]]$Count <- as.numeric(dat[[1]]$Count)
    return(dat)
}

#' Identify source databases from a .tab file
#'
#' Read in a .tab file from the Consensus Pathway Database (CPDB), and
#' identify the source databases present.
#' 
#' @export
#'
#' @param file The filename
#'
#' @return A table of the source databases, with the number of gene sets from
#' each one.

read_cpdb_sourceDBs <- function(file) {
    if (!grepl("\\.tab$", file)[1]) {
        stop("Pathway information must be a .tab file")
    }
    
    geneSetTable <- read.delim(file,
                               stringsAsFactors = FALSE)
    
    return(table(geneSetTable$source))
}

#' Draw volcano plot of differential expression results
#'
#' Draw a volcano plot for results of a differential expression analysis by
#' limma.
#' 
#' @export
#'
#' @param limmaResults Result from runLimmaAnalysis.
#' @param plotContrast Contrast to select for volcano plot. Should be one of the
#' columns in the limma coefficients matrix (for example, a sample group that
#' was compared against the base group, or one of the contrasts in the design
#' matrix). If NULL (default), will plot the first non-Intercept column from
#' the limma coefficients matrix.
#' @param y.var The variable to plot for the y axis, either "p.value" or
#' "q.value" (the false discovery adjusted p-value)
#' 
#' @return A volcano plot using ggplot2
#' 
#' @examples
#' data(ExampleResults) # Results from runLimmaAnalysis
#' 
#' deVolcano(ExampleResults, plotContrast = "Autoimmune.retinopathy")

deVolcano <- function(limmaResults, 
                      plotContrast = NULL, y.var = c("p.value", "q.value")) {
  
    # Bind local variables
    log2FC <- log10p <- NULL
    
    # Identify contrast if not provided
    if (is.null(plotContrast)) {
        plotContrast <-
            colnames(limmaResults$coefficients)[
                which(!(colnames(limmaResults) %in% 
                        c("Intercept", "(Intercept)")))[1]]
      
        cat("\n'plotContrast' not provided, setting it to", plotContrast, "\n")
    }
    
    # Set up data frame for plot
    df <- data.frame(log2FC = limmaResults$coefficients[,plotContrast],
                     log10p = -log10(limmaResults[[y.var[1]]][,plotContrast]))
    
    plt <- ggplot(df, aes(x = log2FC, y = log10p)) +
        geom_point() +
        xlab("log2(Fold Change)") +
        ylab(paste0("-log10(", substr(y.var[1], 1, 1), ")")) +
        theme_bw()
    
    return(plt)
  
}

---
title: "Analyzing NanoString nCounter Data with the NanoTube"
author: "Caleb A Class"
output: rmarkdown::html_vignette
bibliography: NanoTube.bib
vignette: >
    %\VignetteIndexEntry{NanoTube Vignette}
    %\VignetteEngine{knitr::rmarkdown}
    \usepackage[utf8]{inputenc}
---


```{r setup, echo = FALSE, results = "hide"}
knitr::opts_chunk$set(tidy = FALSE,
                      cache = FALSE,
                      dev = "png",
                      message = FALSE, error = FALSE, warning = TRUE,
                      fig.height = 4, fig.width = 4)

library(kableExtra)
```


# Abstract

This library provides tools for the processing, normalization, analysis, and
visualization of NanoString nCounter gene expression data. 
Standard NanoString-suggested analysis steps are supported, and functions
are also provided for interoperability with other published NanoString analysis
methods, as well as a pre-ranked gene set analysis method. This vignette 
provides a simple workflow for nCounter data analysis, as well as more detailed
descriptions of NanoTube functions and options.

# A basic workflow
`processNanostringData` allows you to read in nCounter expression data (from
RCC files or in tabular form) and conduct basic normalization and quality
control checks in one step. We use example data from GEO data series 
GSE117751 [@lundy_t_2018]. For this example, RCC files are provided in 
"GSE117751_RAW", while the sample characteristics table is 
"GSE117751_sample_data.csv".

```{r Basic1}

library(NanoTube)

example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
sample_info <- system.file("extdata", 
                           "GSE117751_sample_data.csv", 
                           package = "NanoTube")

```

A variety of data processing and normalization options are provided in 
`processNanostringData`. A set of default options recommended by nCounter can 
be run automatically, or they can be specified and customized. More details 
are provided in the "Processing Data" section. This function also merges the 
expression data with the sample characteristics table (if provided), and 
outputs as an Biobase ExpressionSet.

```{r Basic2}

dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info,
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis")

```

There are three groups of samples being compared in this data set.

```{r BasicGroups}

table(dat$groups)

```

You can then run differential expression analysis using Limma. Functions are
also provided to allow for analysis using NanoStringDiff (See 'Differential 
expression analysis - Using NanoStringDiff'). For this example, we will compare
the two disease states vs. the control group ("None") by setting `base.group` to
"None".

```{r Basic3}

limmaResults <- runLimmaAnalysis(dat, base.group = "None")

```

DE results can be viewed or exported to a text fileusing `makeDiffExprFile`. 
For example, we can convert the differential expression statistics to a table 
for easier Viewing.

```{r Basic3b}

limmaStats <- makeDiffExprFile(limmaResults, filename = NULL, returns = "stats")
limmaStats <- as.data.frame(limmaStats)

# Rounding for clarity
limmaTab <- head(limmaStats[order(limmaStats$`p-val (Autoimmune.retinopathy)`, 
                      decreasing = FALSE), 1:4])
limmaTab[,1] <- format(limmaTab[,1], digits = 2, nsmall = 1)
limmaTab[,3] <- format(limmaTab[,3], digits = 1, scientific = TRUE)
limmaTab[,4] <- format(limmaTab[,4], digits = 1, nsmall = 1)

# Order by lowest to highest p-value for 'Autoimmune Retinopathy' vs. 'None'
knitr::kable(head(limmaTab), 
      row.names = TRUE, format = "html", align = "c")


```

A volcano plot can also be drawn using deVolcano.

```{r BasicVol}

deVolcano(limmaResults, plotContrast = "Autoimmune.retinopathy")

```

DE results can also be used as input for pre-ranked gene set analysis in the 
`fgsea` package, using `limmaToFGSEA` or `nsdiffToFGSEA`. Gene sets can be 
provided in .gmt, .tab, or .rds (list object) format, or a list can be input 
directly. Plenty of additional options for GSEA analysis are available, 
including leading edge analysis, gene set clustering, and reporting options 
(see 'Gene set analysis').

```{r Basic4}

data("ExamplePathways")

fgseaResults <- limmaToFGSEA(limmaResults, gene.sets = ExamplePathways)

# FGSEA was run separately for Autoimmune Retinopathy vs. None and 
# Retinitis Pigmentosa vs. None
names(fgseaResults)

```

```{r BasicTab}

fgseaTab <- head(fgseaResults$Autoimmune.retinopathy[
             order(fgseaResults$Autoimmune.retinopathy$pval, 
                   decreasing = FALSE),])
fgseaTab[,2:6] <- lapply(fgseaTab[,2:6], format, digits = 1, nsmall = 1)

knitr::kable(fgseaTab, 
      row.names = FALSE, format = "html", align = "c")


```


# Processing Data
## From RCC files
One Reporter Code Count (RCC) file is generated by the nCounter instrument
for each sample, containing the counts for each gene and control in the 
codeset. It also includes some basic quality control (QC) metrics that can
by imported by NanoTube. Two functions are provided in this package: 
`read_rcc`, which reads in a single RCC file, and `read_merge_rcc` which reads
in a vector of RCC file names and combines the data. 

`processNanostringData` includes the reading of these data files contained in `nsFiles`, 
merging with a sample meta-data table `sampleTab`, and other steps described below. 
Ideally, your sampleTab will have a column which contains the names of each respective
RCC file so the expression data and meta-data can be matched properly: this is 
supplied to `processNanostringData` using the `idCol` option, which should be the 
name of the column with the RCC filenames. If this is not provided, the function will
merge your meta-data table in its original order with the RCC files in alphabetical order, and a
warning will be returned advising you to check that the merge was completed correctly (see examples).

`processNanostringData` also conducts optional quality control and normalization 
steps. These are described in the 'Quality Control' and 'Normalization' sections. 
Normalization can be skipped in this function using `normalization = "none"`. 
The housekeeping normalization step can be skipped (retaining positive control 
normalization) using `skip.housekeeping = TRUE`.

```{r proc1}

# Without supplying idCol, a warning is returned.
# The CSV file contains samples in the proper order, so this is technically ok.
dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver",
                             bgType = "t.test", bgPVal = 0.01,
                             skip.housekeeping = FALSE,
                             output.format = "ExpressionSet")

```

```{r proc1idCol}

# With idCol
dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver",
                             bgType = "t.test", bgPVal = 0.01,
                             skip.housekeeping = FALSE,
                             output.format = "ExpressionSet")

```

## From Other files
Expression data can also be imported from a table in a .txt or .csv file, 
possibly produced by the RCC Collector tool.

```{r proc2}

csv_data <- system.file("extdata", 
                        "GSE117751_expression_matrix.csv", 
                        package = "NanoTube")

dat2 <- processNanostringData(nsFile = csv_data,
                             sampleTab = sample_info, 
                             idCol = "GEO_Accession", 
                             groupCol = "Sample_Diagnosis",
                             normalization = "none")

```
## Normalization
This package provides four options for data normalization: a standard set of 
steps recommended by NanoString for nCounter data, which normalizes the data to 
different sets of control genes [@nanostring_technologies_ncounter_2011]; the 
RUV-III (Remove Unwanted Variance III) method, which normalizes the data using 
technical replicates [@gagnon-bartsch_ruv_2019]; the RUVg method, which 
normalizes the data using specified control genes [@bhattacharya_approach_2020]; 
or no normalization.

### nSolver Normalization
To normalize using standard nSolver steps, set `normalization` to "nSolver". 
The three normalization steps under this method are:

1. Positive control normalization: The geometric mean of positive control probes
is used to calculate a scaling factor for each sample.
2. Background assessment: Negative control probes are used to assess whether
endogenous genes are truly expressed above the level of backgrounds. Two options
are provided for this
    + `bgType = "threshold"` - The background threshold will be defined 
    independently for each sample, using 
    `threshold = bgThreshold * sd(NegativeControls) + mean(NegativeControls)`, 
    where `bgThreshold` is set by the user. For each gene, the proportion of 
    samples with expression above the threshold is calculated, and only genes 
    with a proportion greater than `bgProportion` (set by user) will be 
    retained for analysis. For example, if `bgThreshold` is 2 and `bgProportion`
    is 0.5 (defaults), a gene will be retained for analysis if its expression is
    2 standard deviations above the mean of negative control probes in at least
    half of the samples.
    + `bgType = "t.test"` - Expression of each gene is compared with all of the 
    negative control probes (across samples) using a one-sided, two-sample 
    t test. Genes will be retained for analysis if the endogenous gene 
    expression is greater, with p < `bgPVal`.
    + `bgSubtract` - By setting this to TRUE, the calculated background level 
    (`mean + bgThreshold * sd`) will be subtracted from the expression values 
    for each gene in each sample. After subtraction, genes with negative values
    will be set to zero.
3. Housekeeping normalization: The geometric mean of housekeeping gene 
expression is used to calculate a scaling factor for each sample. Housekeeping
genes (symbols or accessions) can be input using the `housekeeping` option. 
If not provided, genes marked as "Housekeeping" in the RCC files will be used. 
Alternatively, this can be skipped using `skip.housekeeping = TRUE`.


```{r norm1}

# Set housekeeping genes manually (optional)
hk.genes <- c("TUBB", "TBP", "POLR2A", "GUSB", "SDHA")

dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver",
                             bgType = "t.test", bgPVal = 0.01,
                             housekeeping = hk.genes, skip.housekeeping = FALSE)

```

```{r norm2}

# Automatically detect housekeeping genes

dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver",
                             bgType = "t.test", bgPVal = 0.01)

```

### RUV-III Normalization

The RUV-III method normalizes data based on technical replicates. Replicate IDs
must be included in the sample information table ('sampleTab' in `processNanostringData`),
and the column containing these IDs is specified using the 'replicateCol' option.
Additionally, the number of dimensions of unwanted data can be specified using
the 'n.unwanted' option.  If NULL (the default), the maximum possible number of
factors will be identified and used.  More information can be found in 
[@gagnon-bartsch_ruv_2019]. 

Note that RUVIII returns log-transformed counts, in contrast to nSolver and
RUVg normalization. This difference is accounted for in subsequent NanoTube 
analysis steps, but it should be considered by the user when using the 
normalized data for other purposes.

```{r normRUVIII}

# This sample data table contains fake replicate identifiers (for use in example).
sample_info_reps <- system.file("extdata", 
                           "GSE117751_sample_data_replicates.csv", 
                           package = "NanoTube")

datRUVIII <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info_reps, 
                             idCol = "RCC_Name",
                             replicateCol = "Replicate_ID",
                             groupCol = "Sample_Diagnosis",
                             normalization = "RUVIII",
                             bgType = "t.test", bgPVal = 0.01,
                             n.unwanted = 1)

```

RUV-III can also be used without technical replicates, by generating PRPS 
(pseudo-replicates of pseudo-samples). This is described in [@molania_removing_2022],
and it was demonstrated to provide superior normalization vs. other RNA-Seq
normalization methods for TCGA RNA-Seq data sets. Software and vignettes for 
accomplishing this normalization can be found at 
<https://github.com/RMolania/TCGA_PanCancer_UnwantedVariation>.

### RUVg Normalization

The RUVg method normalizes data based on housekeeping genes. These can be manually
specified using the 'housekeeping' option in `processNanostringData`, or they can
be automatically interpreted based on the CodeClass column of the RCC files. By
default, RUVg will remove one dimension of unwanted data, but this can be adjusted
using the 'n.unwanted' option. Additionally, the 'RUVg.drop' option can be used
to drop some number of singular values. This defaults to zero, but it can be adjusted
if the first singular value(s) identify the effect of interest. See 
[@bhattacharya_approach_2020] for more details.

```{r normRUVg}

hk.genes <- c("TUBB", "TBP", "POLR2A", "GUSB", "SDHA")

datRUVg <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "RUVg",
                             bgType = "t.test", bgPVal = 0.01,
                             n.unwanted = 1, RUVg.drop = 0,
                             housekeeping = hk.genes)

```

```{r normRUVg2}

# Housekeeping genes are automatically identified
datRUVg <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "RUVg",
                             bgType = "t.test", bgPVal = 0.01,
                             n.unwanted = 1, RUVg.drop = 0)

```


## Assessing Normalization Performance
### RLE Plots
Relative Log Expression (RLE) plots have been demonstrated as effective visualizations to assess whether NanoString data normalization has been successful [@gandolfo_rle_2018]. This method centers the log-expression of each gene to the median log-expression of that gene. The RUV implementation `ruv_rle` can be used to generate this plot, regardless of what normalization method was used [@gagnon-bartsch_ruv_2019].  

Note that for this function, you need to log-transform and transpose the 
expression data matrix.  You'll probably want to set the y-axis limits as well,
using 'ylim'.

```{r rle, fig.width = 6}

# Here's how to do it with the NanoString ExpressionSet. If your data set is in
# list form, you would use dat$exprs instead of exprs(dat).

ruv::ruv_rle(t(log2(exprs(dat))), ylim = c(-2, 2))

```


### Principal components analysis

PCA can be conducted after processing and normalization. We provide a standard
plot using `ggplot2` or an interactive plot using `plotly` (use the 
`interactive.plot` option).

```{r pca, fig.width = 6}

dat <- processNanostringData(example_data, 
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver", 
                             bgType = "t.test", 
                             bgPVal = 0.01)

library(plotly) 
nanostringPCA(dat, interactive.plot = TRUE)$plt

```

We default to plotting the first two principal components, but you can also
choose others.

```{r pca2, fig.width = 6}

nanostringPCA(dat, pc1=3, pc2=4, interactive.plot = FALSE)$plt

```

# Quality Control
NanoTube provides the quality control metrics recommended for NanoString
nCounter data. The raw NanoString data can be loaded for QC using the 
`output.format = "list"` option of `processNanostringData`.

```{r qc1}

dat <- processNanostringData(example_data, 
                             idCol = "RCC_Name",
                             output.format = "list",
                             includeQC = TRUE)

```

Basic QC and cartridge data are loaded in from the RCC files if `includeQC` is 
set to TRUE.

```{r qc2}

head(dat$qc)[,1:5]

```

Positive QC statistics can be calculated and presented as a table. This includes
the positive scaling factors and R-squared values for the expected vs. observed
positive control counts. NanoString recommends positive scaling factors between
0.3 and 3, and R-squared values greater than 0.95. Samples with values outside
these recommendations should be investigated further. 

```{r qcPos1}

posQC <- positiveQC(dat)

knitr::kable(head(posQC$tab), 
      row.names = FALSE, format = "html", align = "c", digits = 2)

```


Positive control genes can be plotted for all samples (default), or a specified
subset of samples (specified by column index, or sample names).

```{r qcPos2, fig.width = 7}

posQC2 <- positiveQC(dat, samples = 1:6)

posQC2$plt

```


Standard negative control statistics can be obtained using the `negativeQC` 
function.


```{r qcNeg1}

negQC <- negativeQC(dat, interactive.plot = FALSE)

knitr::kable(head(negQC$tab), 
      row.names = TRUE, format = "html", align = "c")

```

Negative control genes can also be plotted for each sample.

```{r qcNeg2, fig.height = 7, fig.width = 7}

negQC$plt

```

Housekeeping normalization scale factors can also be obtained from the
processed data. Manufacturer recommends additional caution for samples with
scale factors outside the range of 0.1-10.

```{r qcHK1}
signif(head(dat$hk.scalefactors), digits = 2)
```

# Data Analysis
## Differential expression analysis
### Using Limma

After normalization, the data are likely to resemble a normal distribution,
particularly with reasonable filtering of genes with expression 
below background levels. It is a good idea to 
check this before using limma [@ritchie_limma_2015]. See `limma` vignette for
full details and additional analysis options.

Assuming this information was provided in the `processNanostringData` step, your
ExpressionSet will already contain the sample groups.

```{r gps}

dat <- processNanostringData(example_data, 
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver", 
                             bgType = "t.test", 
                             bgPVal = 0.01)

table(dat$groups)

```

`runLimmaAnalysis` allows you to conduct group-vs-group comparisons by defining
the `base.group`. In this example, setting `base.group` to "None" will cause
Limma to build a linear model fitting the expression data, where the intercept
is the average log2 expression of "None", and the two other coefficients will
correspond to the log2(FC) of Autoimmune retinopathy vs. None and Retinitis 
pigmentosa vs. None.  This function will return standard Limma analysis results.

```{r deLimma}

limmaResults <- runLimmaAnalysis(dat, base.group = "None")

head(signif(limmaResults$coefficients, digits = 2))

```

You can also directly define a design matrix, instead. For example, if this data
were collected in two batches, you could include a batch term in the analysis. 

```{r deLimma2, fig.height = 3.5}

# Generate fake batch labels
batch <- rep(c(0, 1), times = ncol(dat) / 2)

# Reorder groups ("None" first)
group <- factor(dat$groups, 
                levels = c("None", "Autoimmune retinopathy", 
                           "Retinitis pigmentosa"))

# Design matrix including sample group and batch
design <- model.matrix(~group + batch)

# Analyze data
limmaResults2 <- runLimmaAnalysis(dat, design = design)

# We can see there is no batch effect in this data, due to the somewhat uniform
# distribution of p-values. This is good, since the batches are fake.
hist(limmaResults2$p.value[,"batch"], main = "p-values of Batch terms")

```

Alternatively, a design matrix can be provided as 'sampleData' in the 
`processNanostringData` step. This one contains columns for the Intercept,
the two diagnoses, and patient Age.

```{r desInput}

sample_info_design <- system.file("extdata", 
                           "GSE117751_design_matrix.csv", 
                           package = "NanoTube")

datDes <- processNanostringData(example_data, 
                             sampleTab = sample_info_design, 
                             idCol = "RCC_Name",
                             normalization = "nSolver", 
                             bgType = "t.test", 
                             bgPVal = 0.01)

head(pData(datDes))

```

We can then use this pData object as our design matrix in limma analysis. Note that 
`processNanostringData` adds an additional column at the beginning and the end,
so these should be removed for limma analysis. Based on this analysis, many
genes appear to be correlated with patient age.

```{r limma3}

# Analyze data
limmaResults3 <- runLimmaAnalysis(datDes, 
                                  design = pData(datDes)[,2:(ncol(pData(datDes))-1)])


hist(limmaResults3$p.value[,"Age"], main = "p-values for Age term")

```

Results of Limma analyses can be visualized using simple volcano plots.

```{r volcano}

deVolcano(limmaResults, y.var = "p.value")

```

You can add additional ggplot layers as well.

```{r volcano2}

deVolcano(limmaResults, plotContrast = "Autoimmune.retinopathy", 
          y.var = "p.value") +
  geom_hline(yintercept = 2, linetype = "dashed", colour = "darkred") +
  geom_vline(xintercept = 0.5, linetype = "dashed", colour = "darkred") +
  geom_vline(xintercept = -0.5, linetype = "dashed", colour = "darkred")


```


They can also be converted to a simple table or exported to a text file.

```{r deExport}

limmaStats <- makeDiffExprFile(limmaResults, filename = NULL, returns = "stats")
limmaStats <- as.data.frame(limmaStats)

# Order by lowest to highest p-value for 'Autoimmune Retinopathy' vs. 'None'
knitr::kable(head(limmaStats[order(limmaStats$`p-val (Autoimmune.retinopathy)`, 
                                   decreasing = FALSE),]), 
      row.names = TRUE, format = "html", align = "c")


```


### Using NanoStringDiff

NanoStringDiff models the data using a negative binomial approximation, which
has been shown to be generally more accurate for gene expression count data 
[@wang_nanostringdiff_2016]. We have provided a function to convert processed,
unnormalized, data to a `NanoStringSet` for use with NanoStringDiff.

```{r nsDiffConvert}

# Remember to set normalization = "none"
datNoNorm <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             idCol = "RCC_Name",
                             groupCol = "Sample_Diagnosis",
                             normalization = "none")

nsDiffSet <- makeNanoStringSetFromEset(datNoNorm)

colnames(pData(nsDiffSet))

```

Then you can run as described in the NanoStringDiff vignette (see vignette for
full details). Notably, the glm.LRT step is very slow for sample sizes above 10
(and may still be slow with fewer samples). Interested users could also consider
using DESeq2 if a negative binomial method is desired, although that method 
would not explicitly handle the various control genes [@love_moderated_2014].

```{r nsDiffRun, eval = FALSE}

### This block is not run! ###

# This step is fast
nsDiffSet <- NanoStringDiff::estNormalizationFactors(nsDiffSet)

# This step likely to take multiple hours on standard desktop computers.
result <- NanoStringDiff::glm.LRT(nsDiffSet, 
                                  design.full = as.matrix(pData(nsDiffSet)),
                                  contrast = c(1, -1, 0)) 
                                  #Contrast: Autoimmune retinopathy vs. None

```

## Gene set analysis

Gene Set Enrichment Analysis can be conducted on the Limma results using the 
`fgsea` package [@sergushichev_algorithm_2016]. Before running it, it is 
useful to consider whether Gene Set Enrichment Analysis is appropriate for
your specific data set, based on the number and types of genes represented in
your chip, and whether any of them were actually differentially expressed.

Gene set databases can be loaded as a list object (either directly or in an 
.rds file), or as a .gmt (MSigDB or similar) or .tab (CPDB) file. We have 
included a list of pathways from WikiPathways [@martens_wikipathways_2021].  

The `min.set` option is important, as pathways containing only a few genes 
present in your data set will probably not provide informative enrichment 
statistics. We will discard all gene sets where fewer than `min.set` genes from
that set are present in the analysis. You also have the option to rank genes by
'coefficients' (frequently, the log2FC) or the 't' statistics. `skip.first` 
will skip the first column of the limma design if TRUE (default).  Generally, 
this column is the Intercept of the regression, which is not useful for gene 
set analysis.

The `limmaToFGSEA` function will then conduct preranked analysis using `fgsea`
for each column in the Limma coefficients or t-statistic matrix (possibly 
skipping the first).  The output will be a list object containing the results
for each analysis.

```{r gsea1}
data("ExamplePathways")

fgseaResults <- limmaToFGSEA(limmaResults, gene.sets = ExamplePathways, 
                            min.set = 5, rank.by = "t",
                            skip.first = TRUE)


names(fgseaResults)

```

We can order the pathways by p-value and view the top results. As previously
reported in [@lundy_t_2018], we see that immune pathways are significantly
altered in autoimmune retinopathy patients. We also identify EGFR Signaling as a
potential pathway of interest.

```{r gsea2}

fgseaTab <- head(fgseaResults$Autoimmune.retinopathy[
             order(fgseaResults$Autoimmune.retinopathy$pval, 
                   decreasing = FALSE),])
fgseaTab[,2:6] <- lapply(fgseaTab[,2:6], format, digits = 1, nsmall = 1)

knitr::kable(fgseaTab, 
      row.names = FALSE, format = "html", align = "c")


```

Another similar function, `nsdiffToFGSEA`, is provided to conduct fgsea on 
`NanoStringDiff` results. This one conducts analysis on a single preranked list.

After analysis, the leading edge genes can be extracted for gene sets (with 
some cutoff for enrichment statistics) using `fgseaToLEdge`.

```{r gsea3}

# Leading edge for pathways with adjusted p < 0.2
leading.edge <- fgseaToLEdge(fgsea.res = fgseaResults,
                             cutoff.type = "padj", cutoff = 0.2) 

```

The nominal p-value or NES (normalized enrichment score) can also be used as 
a cutoff. If NES is used, you can either select all gene sets with 
abs(NES) > cutoff, if `nes.abs.cutoff == TRUE`. Otherwise, you can select gene 
sets with NES > cutoff (if cutoff > 0) or NES < cutoff (if cutoff < 0).

```{r gsea4}

# Leading edge for pathways with abs(NES) > 1
leading.edge.nes <- fgseaToLEdge(fgsea.res = fgseaResults,
                                cutoff.type = "NES", cutoff = 1,
                                nes.abs.cutoff = TRUE) 

# Leading edge for pathways with NES > 1.5
leading.edge.nes <- fgseaToLEdge(fgsea.res = fgseaResults,
                                cutoff.type = "NES", cutoff = 1.5,
                                nes.abs.cutoff = FALSE) 

# Leading edge for pathways with NES < -0.5
leading.edge.nes <- fgseaToLEdge(fgsea.res = fgseaResults,
                                cutoff.type = "NES", cutoff = -0.5,
                                nes.abs.cutoff = FALSE) 


```

A basic leading edge heatmap can then be drawn using the `pheatmap` package.

```{r gsea5, fig.width = 8, fig.height = 3}

pheatmap::pheatmap(t(leading.edge$Autoimmune.retinopathy),
                   legend = FALSE,
                   color = c("white", "black"))

```

You can further cluster pathways by their leading edge genes. This is 
particularly useful when you have lots (tens to hundreds) of significantly 
enriched pathways, as you can prioritize certain ones or potentially identify
overarching patterns of pathway enrichment. We recommend clustering pathways by
the binary distance (also known as the Jaccard index), which for two gene sets
will be the number of genes shared by the gene sets, divided by the total number
of genes in either of the two gene sets.

```{r gsea6}

# Group pathways with a binary distance below 0.5
fgsea.grouped <- groupFGSEA(fgseaResults$Autoimmune.retinopathy, 
                                   leading.edge$Autoimmune.retinopathy,
                                   join.threshold = 0.5,
                                   dist.method = "binary")

fgseaTab <- head(fgsea.grouped)
fgseaTab[,2:6] <- lapply(fgseaTab[,2:6], format, digits = 1, nsmall = 1)

knitr::kable(fgseaTab, 
      row.names = FALSE, format = "html", align = "c")

```

In this example, only "EGF/EGFR Signaling Pathway" and "Kit receptor signaling
pathway" are sufficiently similar to be clustered together ("Cluster 1"), and
the other four pathways are deemed unique. In the heatmap above, we see that
the Kit receptor pathway contains only one leading edge gene unique from the
EGF/EGFR pathway (PTPN6). Based on a lower p-value and higher NES, you would 
consider EGFR Signaling to be more important in this analysis, while the Kit 
receptor pathway is largely redundant. This is denoted by the "Cluster.Max"
variable, which identifies maximum enrichment in each cluster with an "x".

### Exporting GSEA results

`fgseaPostprocessingXLSX` allows you to output the results of gene set analyses
to an Excel spreadsheet (`fgsePostprocessing` is similar, and provides .txt
files). A summary sheet shows the overall GSEA results, while an additional
table for each separate analysis (A vs. Control, B vs. Control, etc.) shows
the differential expression statistics and expression profiles for leading edge
genes. This step requires input of the FGSEA results, the leading edge results,
and the Limma results. It will cluster the pathways if specified, prior to 
generating results tables.

```{r exportGSEA}

fgseaPostprocessingXLSX(genesetResults = fgseaResults,
                        leadingEdge = leading.edge,
                        limmaResults = limmaResults,
                        join.threshold = 0.5,
                        filename = "analysis.xlsx")

```
You can also use `groupedGSEAtoStackedReport` to generate the gene-level report
for one comparison.

```{r exportGSEA2}

results.AR <- groupedGSEAtoStackedReport( 
    fgsea.grouped,
    leadingEdge = leading.edge$Autoimmune.retinopathy,
    de.fit = limmaResults)

# View Cluster 1 gene statistics
resultsTab <- results.AR[results.AR$Cluster == 1, 1:6]
resultsTab[,2:6] <- lapply(resultsTab[,2:6], format, digits = 1, nsmall = 1)

knitr::kable(resultsTab, 
      row.names = FALSE, format = "html", align = "c")

```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupFGSEA.R
\name{groupFGSEA}
\alias{groupFGSEA}
\title{Cluster gene set analysis results}
\usage{
groupFGSEA(
  gsea.res,
  l.edge,
  join.threshold = NULL,
  ngroups = NULL,
  dist.method = "binary",
  returns = c("signif", "all")
)
}
\arguments{
\item{gsea.res}{Results from pathway analysis for a single comparison, 
using limmaToFSEA.}

\item{l.edge}{Leading edge result from fgseaToLEdge.}

\item{join.threshold}{The threshold distance to join gene sets. Gene sets 
with a distance below this value will be joined to a single "cluster."}

\item{ngroups}{The desired number of gene set groups. Either
'join.threshold' or 'ngroups' must be specified, 'ngroups' takes priority
if both are specified.}

\item{dist.method}{Method for distance calculation (see options for dist()).
We recommend the 'binary' (also known as Jaccard) distance.}

\item{returns}{Either "signif" or "all". This argument defines whether only 
significantly enriched gene sets are included in the output table, or if 
the full results are included. Regardless of this selection, only 
significantly enriched gene sets are clustered.}
}
\value{
A data frame including the FGSEA results, plus two additional 
columns for the clustering results:

\item{Cluster}{The cluster that the gene set was assigned to. Gene 
sets in the same cluster have a distance below the join.threshold.}

\item{best}{Whether the gene set is the most enriched (by p-value) 
in a given cluster.}
}
\description{
Groups the pathway analysis results (using limmaToFGSEA or nsdiffToFGSEA) 
based on the enriched gene sets' leading edges. If the calculated distance 
metric is lower than the given threshold (i.e. the gene sets have highly 
overlapping leading edge genes), these gene sets will be joined to a single 
gene set "cluster." Or if 'ngroups' is specified, gene sets will be clustered
by similarity into that number of groups.
}
\examples{
data("ExamplePathways")
data("ExampleResults") # Results from runLimmaAnalysis

fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways,
                             min.set = 5, rank.by = "t")

leadingEdge <- fgseaToLEdge(fgseaResults, cutoff.type = "padj", 
                            cutoff = 0.25)

# Group the results, and only returns those satisfying the cutoff specified 
# in leadingEdge()
groupedResults <- groupFGSEA(fgseaResults$Autoimmune.retinopathy, 
                             leadingEdge$Autoimmune.retinopathy,
                             join.threshold = 0.5,
                             returns = "signif")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalize_pos_controls.R
\name{normalize_pos_controls}
\alias{normalize_pos_controls}
\title{Positive control gene normalization}
\usage{
normalize_pos_controls(dat, logfile = "")
}
\arguments{
\item{dat}{NanoString data, including expression matrix and gene dictionary.}

\item{logfile}{Optional name of logfile to print messages, warnings or 
errors.}
}
\value{
NanoString data, with expression matrix now normalized by 
positive control gene expression.
}
\description{
Scale genes by the geometric mean of positive control genes. This step is
conducted within processNanostringData, when normalization is set to 
"nCounter".
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")

dat <- read_merge_rcc(list.files(example_data, full.names = TRUE))

# Positive controls are identified in the RCC files, and used to 
# normalize the data
dat <- normalize_pos_controls(dat)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_cpdb_tab.R
\name{read_cpdb_tab}
\alias{read_cpdb_tab}
\title{Read .tab file.}
\usage{
read_cpdb_tab(file, sourceDB = NULL)
}
\arguments{
\item{file}{The filename}

\item{sourceDB}{The source database to use. If NULL (default), retains
gene sets from all source databases}
}
\value{
A list object, containing a character vector of genes for
each gene set.
}
\description{
Read in a .tab file from the Consensus Pathway Database (CPDB)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fgseaToLEdge.R
\name{fgseaToLEdge}
\alias{fgseaToLEdge}
\title{Generate leading edge matrix from fgsea results.}
\usage{
fgseaToLEdge(
  fgsea.res,
  cutoff.type = c("padj", "pval", "NES", "none"),
  cutoff = 0.05,
  nes.abs.cutoff = TRUE
)
}
\arguments{
\item{fgsea.res}{Result from limmaToFGSEA}

\item{cutoff.type}{Filter gene sets by adjusted p-value ('padj'), nominal
p-value ('pval'), normalized enrichment score ('NES'), or include all gene
sets ('none')}

\item{cutoff}{Numeric cutoff for filtering (not used if 
cutoff.type == "none")}

\item{nes.abs.cutoff}{If cutoff.type == "NES", should we use extreme positive
and negative values (TRUE), or only filter in the positive or negative 
direction (FALSE). If TRUE, will select gene sets with abs(NES) > cutoff. 
If FALSE, will select gene sets with NES > cutoff (if cutoff >= 0) or
NES < cutoff (if cutoff < 0)}
}
\value{
a list containing the leading edge matrix for each comparison
}
\description{
Extract leading edge genes from gene sets identified in fgsea analysis.
Gene sets may be filtered by significance or NES.
}
\examples{
data("ExamplePathways")
data("ExampleResults") # Results from runLimmaAnalysis

fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways)

# Generate the leading edge for pathways with padj < 0.25
leadingEdge <- fgseaToLEdge(fgseaResults, 
                            cutoff.type = "padj", cutoff = 0.25)

# Generate the leading edge for pathways with abs(NES) > 2
leadingEdge <- fgseaToLEdge(fgseaResults, cutoff.type = "NES",
                            cutoff = 2, nes.abs.cutoff = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unzip_dirs.R
\name{unzip_dirs}
\alias{unzip_dirs}
\title{Unzip}
\usage{
unzip_dirs(fileDirs)
}
\arguments{
\item{fileDirs}{character list of zip files}
}
\value{
Names of now-unzipped directories
}
\description{
Unzips provided list of directories
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeFGSEAmasterTable.R
\name{makeFGSEAmasterTable}
\alias{makeFGSEAmasterTable}
\title{Make master table of all GSEA results}
\usage{
makeFGSEAmasterTable(
  genesetResults,
  leadingEdge,
  join.threshold = 0.5,
  ngroups = NULL,
  dist.method = "binary",
  filename = NULL
)
}
\arguments{
\item{genesetResults}{Results from pathway analysis using limmaToFGSEA.}

\item{leadingEdge}{Results from fgseaToLEdge}

\item{join.threshold}{The threshold distance to join gene sets. Gene sets 
with a distance below this value will be joined to a single "cluster."}

\item{ngroups}{The desired number of gene set groups. Either 
'join.threshold' or 'ngroups' must be specified, 'ngroups' takes priority 
if both are specified.}

\item{dist.method}{Method for distance calculation (see options for dist()).
We recommend the 'binary' (also known as Jaccard) distance.}

\item{filename}{File name for the output text file. If NULL (default), data 
will be returned as an R data frame.}
}
\value{
A table of GSEA results, clustered by similarity of leading edge.
}
\description{
This function clusters GSEA results by leading edge similarity, and then 
combines to a data frame or text file.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/untar_dirs.R
\name{untar_dirs}
\alias{untar_dirs}
\title{Untar}
\usage{
untar_dirs(fileDirs)
}
\arguments{
\item{fileDirs}{character list of tar files}
}
\value{
Names of now-untarred directories
}
\description{
Untars provided list of directories (analogous to unzip_dirs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NanoTube.R
\docType{data}
\name{ExampleResults}
\alias{ExampleResults}
\title{Example results from runLimmaAnalysis}
\format{
An MArrayLM object from limma
}
\usage{
data(ExampleResults)
}
\description{
Results of runLimmaAnalysis using the example data set GSE117751 
(in extdata).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fgseaPostprocessingXLSX.R
\name{fgseaPostprocessingXLSX}
\alias{fgseaPostprocessingXLSX}
\title{Postprocessing for GSEA analyses for Excel}
\usage{
fgseaPostprocessingXLSX(
  genesetResults,
  leadingEdge,
  limmaResults,
  join.threshold = 0.5,
  ngroups = NULL,
  dist.method = "binary",
  filename
)
}
\arguments{
\item{genesetResults}{Results from pathway analysis using limmaToFGSEA.}

\item{leadingEdge}{Results from fgseaToLEdge}

\item{limmaResults}{Results from runLimmaAnalysis}

\item{join.threshold}{The threshold distance to join gene sets. Gene sets 
with a distance below this value will be joined to a single "cluster."}

\item{ngroups}{The desired number of gene set groups. Either 
'join.threshold' or 'ngroups' must be specified, 'ngroups' takes priority 
if both are specified.}

\item{dist.method}{Method for distance calculation (see options for dist()).
We recommend the 'binary' (also known as Jaccard) distance.}

\item{filename}{File name for the output Excel file.}
}
\value{
An Excel file where the first sheet summarizes the gene set analysis 
results. Subsequent sheets are reports showing differential expression 
statistics of leading edge genes.
}
\description{
Clusters GSEA results by leading edge genes, and writes reports showing
gene expression profiles of these genes (to Excel).
}
\examples{
data("ExamplePathways")
data("ExampleResults") # Results from runLimmaAnalysis

fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways)

leadingEdge <- fgseaToLEdge(fgseaResults, cutoff.type = "padj", cutoff = 0.1)

\donttest{
fgseaPostprocessingXLSX(fgseaResults, leadingEdge, 
                    limmaResults = ExampleResults,
                    join.threshold = 0.5,
                    filename = "Results.xlsx")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_cpdb_sourceDBs.R
\name{read_cpdb_sourceDBs}
\alias{read_cpdb_sourceDBs}
\title{Identify source databases from a .tab file}
\usage{
read_cpdb_sourceDBs(file)
}
\arguments{
\item{file}{The filename}
}
\value{
A table of the source databases, with the number of gene sets from
each one.
}
\description{
Read in a .tab file from the Consensus Pathway Database (CPDB), and
identify the source databases present.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupedGSEAtoStackedReport.R
\name{groupedGSEAtoStackedReport}
\alias{groupedGSEAtoStackedReport}
\title{Build a report from gene set enrichment results.}
\usage{
groupedGSEAtoStackedReport(grouped.gsea, leadingEdge, de.fit, outputDir = NULL)
}
\arguments{
\item{grouped.gsea}{Output from groupFGSEA()}

\item{leadingEdge}{Leading edge analysis results used in groupFGSEA()}

\item{de.fit}{Differential Expression results from Limma or NanoStringDiff}

\item{outputDir}{Directory for output files. If NULL (default), will return
the stacked report instead of writing to a file.}
}
\value{
A stacked report containing statistics and gene expression profiles
for genes contained in each cluster
}
\description{
After clustering FGSEA results by gene set similarity, this function builds
a report containing the individual gene expression profiles for genes 
contained in each gene set cluster.
}
\examples{
data("ExamplePathways")
data("ExampleResults") # Results from runLimmaAnalysis

fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways,
                             min.set = 5, rank.by = "t")
leadingEdge <- fgseaToLEdge(fgseaResults, cutoff.type = "padj", cutoff = 0.1)

fgseaGrouped <- groupFGSEA(fgseaResults$Autoimmune.retinopathy, 
                            leadingEdge$Autoimmune.retinopathy,
                            join.threshold = 0.5,
                            dist.method = "binary")

results.AR <- groupedGSEAtoStackedReport(
              fgseaGrouped,
              leadingEdge = leadingEdge$Autoimmune.retinopathy,
              de.fit = ExampleResults)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeDiffExprFile.R
\name{makeDiffExprFile}
\alias{makeDiffExprFile}
\title{Make differential expression results file.}
\usage{
makeDiffExprFile(
  limmaResults,
  filename = NULL,
  returns = c("all", "stats"),
  skip.first = TRUE
)
}
\arguments{
\item{limmaResults}{Result from runLimmaAnalysis}

\item{filename}{The desired name for the output tab-delimited text file. If 
NULL (default) the resulting table will be returned as an R data frame.}

\item{returns}{If "all" (default), will center the log-expression
data on median of base.group expression and include the expression data in 
the output. If "stats", will only include the differential expression 
statistics.}

\item{skip.first}{Logical: Skip the first factor for gene set analysis?
Frequently the first factor is the 'Intercept', which is generally 
uninteresting for GSEA (default TRUE).}
}
\value{
A table of differential expression results
}
\description{
Make a data frame or text file containing coefficients, p-, and q-values 
from Limma differential expression analysis. If returns == "all", will also 
center the log-expression data on the median of base.group expression, and 
include the expression data in the output.
}
\examples{
data("ExampleResults") # Results from runLimmaAnalysis

# Include expression data in the results table
deResults <- makeDiffExprFile(ExampleResults, returns = "all")

# Only include statistics, and save to a .txt file
\donttest{
makeDiffExprFile(ExampleResults, file = "DE.txt",
                 returns = "stats")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nsdiffToFGSEA.R
\name{nsdiffToFGSEA}
\alias{nsdiffToFGSEA}
\title{Run gene set enrichment analysis using DE results.}
\usage{
nsdiffToFGSEA(deResults, gene.sets, sourceDB = NULL, min.set = 1)
}
\arguments{
\item{deResults}{Result from NanoStringDiff::glm.LRT.}

\item{gene.sets}{Gene set file name, in .rds (list), .gmt, or .tab format;
or a list object containing the gene sets. Gene names must be
in the same form as in the ranked.list.}

\item{sourceDB}{Source database to include, only if using a .tab-format 
geneset.file from CPDB.}

\item{min.set}{Number of genes required to conduct analysis on a given gene 
set (default = 1). If fewer than this number of genes from limmaResults are 
included in a gene set, that gene set will be skipped for this analysis.}
}
\value{
A list containing data frames with the fgsea results.
}
\description{
Use the fgsea library to run gene set enrichment analysis from the 
NanoStringDiff analysis results. Genes will be ranked by their log2 fold
changes.
}
\examples{

\donttest{
 
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
sample_data <- system.file("extdata", "GSE117751_sample_data.csv", 
                           package = "NanoTube")

datNoNorm <- processNanostringData(nsFiles = example_data,
                                   sampleTab = sample_data, 
                                   groupCol = "Sample_Diagnosis",
                                   normalization = "none")

# Convert to NanoString Set, retaining 2 samples per group for this example
# (will run faster, but still pretty slow)
nsDiffSet <- makeNanoStringSetFromEset(datNoNorm[,c(1,2,15,16,29,30)])

# Run NanoStringDiff analysis
nsDiffSet <- NanoStringDiff::estNormalizationFactors(nsDiffSet)
result <- NanoStringDiff::glm.LRT(nsDiffSet, 
                                  design.full = as.matrix(pData(nsDiffSet)),
                                  contrast = c(1, -1, 0)) 
                                  #contrast: Autoimmune retinopathy vs. None

# FGSEA with example pathways, only for pathways with at least 5 genes
# analyzed in NanoString experiment
data("ExamplePathways")
fgseaResult <- nsdiffToFGSEA(result, gene.sets = ExamplePathways,
                             min.set = 5)


}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/processNanostringData.R
\name{processNanostringData}
\alias{processNanostringData}
\title{Process NanoString nCounter gene expression data.}
\usage{
processNanostringData(
  nsFiles,
  sampleTab = NULL,
  idCol = NULL,
  groupCol = NULL,
  replicateCol = NULL,
  normalization = c("nSolver", "RUVIII", "RUVg", "none"),
  bgType = c("threshold", "t.test", "none"),
  bgThreshold = 2,
  bgProportion = 0.5,
  bgPVal = 0.001,
  bgSubtract = FALSE,
  n.unwanted = NULL,
  RUVg.drop = 0,
  housekeeping = NULL,
  skip.housekeeping = FALSE,
  includeQC = FALSE,
  sampIds = NULL,
  output.format = c("ExpressionSet", "list"),
  logfile = ""
)
}
\arguments{
\item{nsFiles}{file path (or zip file) containing the .rcc files, or multiple
directories in a character vector, or a single text/csv file containing the 
combined counts, or .rcc files in a character vector.}

\item{sampleTab}{.txt (tab-delimited) or .csv (comma-delimited) file 
containing sample data table (optional, default NULL)}

\item{idCol}{the column name of the sample identifiers in the sample table,
which should correspond to the column names in the count table 
(default NULL: will assume the first column contains the sample identifiers)}

\item{groupCol}{the column name of the group identifiers in the sample table.}

\item{replicateCol}{the column name of the technical replicate identifiers 
(default NULL). Multiple replicates of the same sample will have the same 
value in this column. Replicates are used to improve normalization 
performance in the "RUVIII" method; otherwise they are averaged.}

\item{normalization}{If "nSolver" (default), continues with background, 
positive control, and housekeeping control normalization steps to return
a NanoStringSet of normalized data. If "RUVIII", runs RUV normalization using 
controls, housekeeping genes and technical replicates. If "RUVg", runs RUV 
normalization using housekeeping genes. If "none", returns a 
NanoStringSet with the raw counts, suitable for running NanoStringDiff.}

\item{bgType}{(Only if normalization is not "none") Type of background 
correction to use: "threshold" sets a threshold for N standard deviations 
above the mean of negative controls. "t.test" conducts a one-sided t test 
for each gene against all negative controls. "none" to skip background removal}

\item{bgThreshold}{If bgType=="threshold", number of sd's above the mean to 
set as threshold for background correction.}

\item{bgProportion}{If bgType=="threshold", proportion of samples that a gene
must be above threshold to be included in analysis.}

\item{bgPVal}{If bgType=="t.test", p-value threshold to use for gene to be 
included in analysis.}

\item{bgSubtract}{Should calculated background levels be subtracted from 
reported expressions? If TRUE, will subtract mean+numSD*sd of the negative 
controls from the endogenous genes, and then set negative values to zero 
(default FALSE)}

\item{n.unwanted}{The number of unwanted factors to use (for RUVIII or RUVg
normalization only). If NULL (default), the maximum possible value will
be identified and used.}

\item{RUVg.drop}{The number of singular values to drop for RUVg normalization
(see RUVSeq::RUVg)}

\item{housekeeping}{vector of genes (symbols or accession) to use for 
housekeeping correction ("nCounter" or "RUVg" normalization). 
If NULL, will use genes listed as "Housekeeping" under CodeClass.}

\item{skip.housekeeping}{Skip housekeeping normalization? (default FALSE)}

\item{includeQC}{Should we include the QC from the .rcc files? This can 
cause errors, particularly when reading in files from multiple experiments.}

\item{sampIds}{a vector of sample identifiers, important if there are 
technical replicates. Currently, this function averages technical replicates.
sampIds will be extracted from the replicateCol in the sampleTab, if 
provided.}

\item{output.format}{If "list", will return the normalized (optional) and raw
expression data, as well as various QC and relevant information tables. If 
"ExpressionSet" (default), will convert to an n*p ExpressionSet, with n rows
representing genes and p columns representing samples. ExpressionSet objects
are required for some steps, such as runLimmaAnalysis.}

\item{logfile}{a filename for the logfile (optional). If blank, will print 
warnings to screen.}
}
\value{
An list or ExpressionSet containing the raw and/or normalized 
counts, dictionary, and sample info if provided
}
\description{
This function reads in a zip file or folder containing multiple .rcc files 
(or a txt/csv file containing raw count data), and then optionally conducts 
positive control normalization, background correction, and housekeeping 
normalization.
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
sample_data <- system.file("extdata", "GSE117751_sample_data.csv", 
                           package = "NanoTube")

# Process NanoString data from RCC files present in example_data folder.
# Use standard nCounter normalization, removing genes that do
# pass a t test against negative control genes with p < 0.05. Return the
# result as an "ExpressionSet".

dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_data, 
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver",
                             bgType = "t.test", bgPVal = 0.01,
                             output.format = "ExpressionSet")

# Load NanoString data from a csv file (from NanoString's RCC Collector tool,
# for example). Skip normalization by setting 'normalization = "none"'.

csv_data <- system.file("extdata", "GSE117751_expression_matrix.csv", 
                        package = "NanoTube")
dat <- processNanostringData(nsFile = csv_data,
                              sampleTab = sample_data, 
                              idCol = "GEO_Accession", 
                              groupCol = "Sample_Diagnosis",
                              normalization = "none")
                              
# Load NanoString data from RCC files, using a threshold background level for
# removing low-expressed genes. Also, specify which genes to use for 
# housekeeping normalization. Save the result in "list" format (useful for
# some QC functions) instead of an "ExpressionSet".

dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_data, 
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver",
                             bgType = "threshold", 
                             bgThreshold = 2, bgProportion = 0.5,
                             housekeeping = c("TUBB", "TBP", "POLR2A", 
                                              "GUSB", "SDHA"),
                             output.format = "list")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeNanoStringSetFromEset.R
\name{makeNanoStringSetFromEset}
\alias{makeNanoStringSetFromEset}
\title{Convert NanoString ExpressionSet to NanoStringSet}
\usage{
makeNanoStringSetFromEset(eset, designs = NULL)
}
\arguments{
\item{eset}{NanoString data ExpressionSet, from processNanostringData}

\item{designs}{Design matrix. If NULL, will look for "groups" column in 
pData(eset).}
}
\value{
A NanoStringSet for NanoStringDiff
}
\description{
Convert ExpressionSet from processNanoStringData to a NanoStringSet for use
with the NanoStringDiff package.
}
\examples{
# Example data
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
sample_data <- system.file("extdata", "GSE117751_sample_data.csv", 
package = "NanoTube")

# Load data without normalization
dat <- processNanostringData(nsFiles = example_data,
                     sampleTab = sample_data, groupCol = "Sample_Diagnosis",
                     normalization = "none")
                     
# Convert to NanoStringSet
dat.ns <- makeNanoStringSetFromEset(dat)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalize_housekeeping.R
\name{normalize_housekeeping}
\alias{normalize_housekeeping}
\title{Housekeeping gene normalization}
\usage{
normalize_housekeeping(dat, genes = NULL, logfile = "")
}
\arguments{
\item{dat}{NanoString data, including expression matrix and gene dictionary.}

\item{genes}{List of housekeeping genes to use for normalization. If NULL 
(default), will use all genes marked as "Housekeeping" in codeset.}

\item{logfile}{Optional name of logfile to print messages, warnings 
or errors.}
}
\value{
NanoString data, with expression matrix now normalized by 
housekeeping gene expression.
}
\description{
Scale endogenous and housekeeping genes by the geometric mean of 
housekeeping genes. This should be conducted after positive control 
normalization and background correction.  This step is conducted 
within processNanostringData, when normalization is set to "nCounter".
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")

# Load data, positive control normalization, and background filtering
dat <- read_merge_rcc(list.files(example_data, full.names = TRUE))
dat <- normalize_pos_controls(dat)
dat <- remove_background(dat, mode = "t.test", pval = 0.05)

# Normalize by genes marked "Housekeeping" in RCC files
dat <- normalize_housekeeping(dat)

# Normalize by specified housekeeping genes (gene symbol or accession)
dat <- normalize_housekeeping(dat,
                       genes = c("TUBB", "TBP", "POLR2A", "GUSB", "SDHA"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NanoTube.R
\docType{package}
\name{NanoTube}
\alias{NanoTube}
\title{NanoTube}
\description{
A package for NanoString nCounter gene expression data processing,
analysis, and visualization.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/limmaToFGSEA.R
\name{limmaToFGSEA}
\alias{limmaToFGSEA}
\title{Run gene set enrichment analysis using DE results.}
\usage{
limmaToFGSEA(
  limmaResults,
  gene.sets,
  sourceDB = NULL,
  min.set = 1,
  rank.by = c("coefficients", "t"),
  skip.first = TRUE
)
}
\arguments{
\item{limmaResults}{Result from runLimmaAnalysis.}

\item{gene.sets}{Gene set file name, in .rds (list), .gmt, or .tab format;
or a list object containing the gene sets. Gene names must be
in the same form as in the ranked.list.}

\item{sourceDB}{Source database to include (only if using a .tab-format 
geneset.file from CPDB).}

\item{min.set}{Number of genes required to conduct analysis on a given gene 
set (default = 1). If fewer than this number of genes from limmaResults are 
included in a gene set, that gene set will be skipped for this analysis.}

\item{rank.by}{Rank genes by log2 fold changes ('coefficients', default) or 
t-statistics ('t').}

\item{skip.first}{Logical: Skip the first factor for gene set analysis?
Frequently the first factor is the 'Intercept', which is generally 
uninteresting for GSEA (default TRUE).}
}
\value{
A list containing data frames with the fgsea results for each 
comparison.
}
\description{
Use the fgsea library to run gene set enrichment analysis from the Limma
analysis results. Genes will be ranked by their log2 fold changes
or t-statistics (specified using 'rank.by').
}
\details{
Limma returns matrices of coefficients and t statistics with 
columns for each column in the design matrix. This function will conduct a
separate enrichment analysis on each column from the relevant matrix. Because
the first column may be an "intercept" term, which is generally not relevant
for enrichment analysis, the user may want to skip analysis for that term
(using skip.first = TRUE, the default).
}
\examples{
data("ExamplePathways")
data("ExampleResults") # Results from runLimmaAnalysis

# Use the default settings
fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways)

# Only include gene sets with at least 5 genes in the NanoString data set,
# and rank genes by their "t" statistics.
fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways,
                             min.set = 5, rank.by = "t")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deVolcano.R
\name{deVolcano}
\alias{deVolcano}
\title{Draw volcano plot of differential expression results}
\usage{
deVolcano(limmaResults, plotContrast = NULL, y.var = c("p.value", "q.value"))
}
\arguments{
\item{limmaResults}{Result from runLimmaAnalysis.}

\item{plotContrast}{Contrast to select for volcano plot. Should be one of the
columns in the limma coefficients matrix (for example, a sample group that
was compared against the base group, or one of the contrasts in the design
matrix). If NULL (default), will plot the first non-Intercept column from
the limma coefficients matrix.}

\item{y.var}{The variable to plot for the y axis, either "p.value" or
"q.value" (the false discovery adjusted p-value)}
}
\value{
A volcano plot using ggplot2
}
\description{
Draw a volcano plot for results of a differential expression analysis by
limma.
}
\examples{
data(ExampleResults) # Results from runLimmaAnalysis

deVolcano(ExampleResults, plotContrast = "Autoimmune.retinopathy")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negativeQC.R
\name{negativeQC}
\alias{negativeQC}
\title{Calculate negative control statistics}
\usage{
negativeQC(ns, interactive.plot = FALSE)
}
\arguments{
\item{ns}{NanoString data, processed by `processNanostringData` with
output.format set to 'list' and 'nSolver' normalization.}

\item{interactive.plot}{Generate an interactive plot using plotly? Only 
recommended for fewer than 20 samples (default FALSE)}
}
\value{
A list object containing:
\item{tab}{The table of negative control statistics, including the mean
& standard deviation of negative control genes, calculated background
threshold, and number of endogenous genes below that threshold}
\item{plt}{An object containing the negative control plots.}
}
\description{
Provide a table the negative control statistics, and plot the counts of 
negative control genes in each sample.
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
sample_data <- system.file("extdata", "GSE117751_sample_data.csv", 
                           package = "NanoTube")

# Process and normalize data first
dat <- processNanostringData(example_data, 
                             sampleTab = sample_data, 
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver", 
                             bgType = "threshold", 
                             bgThreshold = 2, bgProportion = 0.5,
                             output.format = "list")

negQC <- negativeQC(dat, interactive.plot = FALSE) 

# View negative QC table & plot
head(negQC$tab)
negQC$plt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fgseaPostprocessing.R
\name{fgseaPostprocessing}
\alias{fgseaPostprocessing}
\title{Postprocessing for GSEA analyses}
\usage{
fgseaPostprocessing(
  genesetResults,
  leadingEdge,
  limmaResults,
  join.threshold = 0.5,
  ngroups = NULL,
  dist.method = "binary",
  reportDir
)
}
\arguments{
\item{genesetResults}{Results from pathway analysis using limmaToFGSEA.}

\item{leadingEdge}{Results from fgseaToLEdge}

\item{limmaResults}{Results from runLimmaAnalysis}

\item{join.threshold}{The threshold distance to join gene sets. Gene sets 
with a distance below this value will be joined to a single "cluster."}

\item{ngroups}{The desired number of gene set groups. Either 
'join.threshold' or 'ngroups' must be specified, 'ngroups' takes priority 
if both are specified.}

\item{dist.method}{Method for distance calculation (see options for dist()).
We recommend the 'binary' (also known as Jaccard) distance.}

\item{reportDir}{Directory for the GSEA reports (each comparison will be
a separate txt file). Directory will be created if it does not exist.}
}
\value{
A table of gene set analysis results, as well as reports showing
differential expression of leading edge genes.
}
\description{
Clusters GSEA results by leading edge genes, and writes reports showing
gene expression profiles of these genes.
}
\examples{
data("ExamplePathways")
data("ExampleResults") # Results from runLimmaAnalysis

fgseaResults <- limmaToFGSEA(ExampleResults, gene.sets = ExamplePathways)

leadingEdge <- fgseaToLEdge(fgseaResults, cutoff.type = "padj", cutoff = 0.1)

\donttest{
fgseaPostprocessing(fgseaResults, leadingEdge, 
                    limmaResults = ExampleResults,
                    join.threshold = 0.5,
                    reportDir = "GSEAresults")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_rcc.R
\name{read_rcc}
\alias{read_rcc}
\title{Read .rcc file}
\usage{
read_rcc(file)
}
\arguments{
\item{file}{file name}
}
\value{
list containing expression data, sample attributes, and basic qc from
the .rcc file.
}
\description{
This function reads in a single .rcc file and splits into expression, 
sample data, and qc components.
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")

# First file only
single_file <- list.files(example_data, full.names = TRUE)[1]
single_dat <- read_rcc(single_file)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanostringPCA.R
\name{nanostringPCA}
\alias{nanostringPCA}
\title{Plot PCA}
\usage{
nanostringPCA(
  ns,
  pc1 = 1,
  pc2 = 2,
  interactive.plot = FALSE,
  exclude.zeros = TRUE,
  codeclass.retain = "endogenous"
)
}
\arguments{
\item{ns}{Processed NanoString data}

\item{pc1}{Principal component to plot on x-axis (default 1)}

\item{pc2}{Principal component to plot on y-axis (default 2)}

\item{interactive.plot}{Plot using plotly? Default FALSE (in which case
ggplot2 is used)}

\item{exclude.zeros}{Exclude genes that are not detected in all samples
(default TRUE)}

\item{codeclass.retain}{The CodeClasses to retain for principal components 
analysis.Generally we're interested in endogenous genes, so we keep 
"endogenous" only by default. Others can be included by entering a character 
vector for this option. Alternatively, all targets can be retained by 
setting this option to ".".}
}
\value{
A list containing:
\item{pca}{The PCA object}
\item{plt}{The PCA plot}
}
\description{
Conduct principal components analysis and plot the results, using either
ggplot2 or plotly.
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
sample_data <- system.file("extdata", "GSE117751_sample_data.csv", 
                           package = "NanoTube")

# Process and normalize data first
dat <- processNanostringData(example_data, 
                             sampleTab = sample_data, 
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver", 
                             bgType = "t.test", bgPVal = 0.01)
                               
# Interactive PCA using plotly                             
nanostringPCA(dat, interactive.plot = TRUE)$plt

# Static plot using ggplot2, for the 3rd and 4th PC's.
nanostringPCA(dat, pc1 = 3, pc2 = 4, interactive.plot = FALSE)$plt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NanoTube.R
\docType{data}
\name{ExamplePathways}
\alias{ExamplePathways}
\title{Example pathway database}
\format{
A list object with 30 vectors of gene symbols, for 30 pathways
}
\usage{
data(ExamplePathways)
}
\description{
A list object containing example gene sets from WikiPathways.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_sampleData.R
\name{read_sampleData}
\alias{read_sampleData}
\title{Read in a sample data table.}
\usage{
read_sampleData(dat, file.name, idCol = NULL, groupCol, replicateCol = NULL)
}
\arguments{
\item{dat}{expression data, read in by read_merge_rcc or read.delim}

\item{file.name}{the path/name of the .txt or .csv file}

\item{idCol}{the column name of the sample identifiers in the sample table,
which should correspond to the column names in the count table 
(default NULL: will assume the first column contains the sample identifiers).}

\item{groupCol}{the column name of the group identifiers.}

\item{replicateCol}{the column name of the replicate identifiers (default
NULL). Multiple replicates of the same sample will have the same value in 
this column.}
}
\value{
The list with the expression data, now combined with the sample
information
}
\description{
Read in a .txt or .csv file containing sample names, group identifiers,
replicate identifiers, and any other sample data. Sample names must be in the
first column and must correspond with sample names in the count data file(s).
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
sample_info <- system.file("extdata", "GSE117751_sample_data.csv", 
                           package = "NanoTube")

dat <- read_merge_rcc(list.files(example_data, full.names = TRUE))

# Merge expression data with sample info
dat <- read_sampleData(dat, file.name = sample_info,
                       groupCol = "Sample_Diagnosis")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runLimmaAnalysis.R
\name{runLimmaAnalysis}
\alias{runLimmaAnalysis}
\title{Conduct differential expression analysis}
\usage{
runLimmaAnalysis(
  dat,
  groups = NULL,
  base.group = NULL,
  design = NULL,
  codeclass.retain = "endogenous",
  ...
)
}
\arguments{
\item{dat}{NanoString data ExpressionSet, from processNanostringData}

\item{groups}{character vector, in same order as the samples in dat. NULL
if already included in 'dat'}

\item{base.group}{the group against which other groups are compared (must
be one of the levels in 'groups'). Will use the first group if NULL.}

\item{design}{a design matrix for Limma analysis (default NULL, will do 
analysis based on provided 'group' data)}

\item{codeclass.retain}{The CodeClasses to retain for Limma analysis. 
Generally we're interested in endogenous genes, so we keep "endogenous" 
only by default. Others can be included by entering a character vector for 
this option (see limmaResults3 example). Alternatively, all targets can be 
retained by setting this option to ".".}

\item{...}{Optional arguments to be passed to limma::lmFit}
}
\value{
The fit Limma object
}
\description{
Use Limma to conduct a simple differential expression analysis. All groups 
are compared against the base.group, and empirical Bayes method is used to
identify significantly differentially expressed genes. Alternatively, a
design matrix can be supplied, as explained in limma::limmaUsersGuide()
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
sample_info <- system.file("extdata", "GSE117751_sample_data.csv", 
                           package = "NanoTube")

dat <- processNanostringData(nsFiles = example_data,
                             sampleTab = sample_info, 
                             groupCol = "Sample_Diagnosis")

# Compare the two diseases against healthy controls ("None")
limmaResults <- runLimmaAnalysis(dat, base.group = "None")


# You can also supply a design matrix
# Generate fake batch labels
batch <- rep(c(0, 1), times = ncol(dat) / 2)

# Reorder groups ("None" first)
group <- factor(dat$groups, levels = c("None", "Autoimmune retinopathy", 
                                       "Retinitis pigmentosa"))

# Design matrix including sample group and batch
design <- model.matrix(~group + batch)

# Analyze data
limmaResults2 <- runLimmaAnalysis(dat, design = design)

# Run Limma analysis including endogenous *and* housekeeping genes.
limmaResults3 <- runLimmaAnalysis(dat, design = design,
                     codeclass.retain = c("endogenous", "housekeeping"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_background.R
\name{remove_background}
\alias{remove_background}
\title{Assess background expression}
\usage{
remove_background(
  dat,
  mode = c("threshold", "t.test"),
  numSD,
  proportionReq,
  pval,
  subtract = FALSE
)
}
\arguments{
\item{dat}{Positive control-scaled NanoString data}

\item{mode}{Either "threshold" (default) or "t.test". If "threshold", 
requires proportionReq of samples to have expression numSD standard 
deviations among the mean of negative control genes. If "t.test", each gene 
will be compared with all negative control genes in a one-sided two-sample 
t-test.}

\item{numSD}{Number of standard deviations above mean of negative control 
genes to used as background threshold for each sample: 
mean(negative_controls) + numSD * sd(negative_controls). 
Required if mode == "threshold" or subtract == TRUE}

\item{proportionReq}{Required proportion of sample expressions exceeding the
sample background threshold to include gene in further analysis. Required if
mode == "threshold" or subtract == TRUE}

\item{pval}{p-value (from one-sided t-test) threshold to declare gene 
expression above background expression level. Genes with p-values above 
this level are removed from further analysis. Required if mode == "t.test"}

\item{subtract}{Should calculated background levels be subtracted from 
reported expressions? If TRUE, will subtract mean+numSD*sd of the negative 
controls from the endogenous genes, and then set negative values to zero 
(default FALSE).}
}
\value{
NanoString data, with genes removed that fail the comparison test
against negative control genes. Expression levels are updated for all genes
if subtract == TRUE.
}
\description{
Compare endogenous gene expression data against negative control genes and
remove data for genes that fail the comparison. This step is
conducted within processNanostringData, when normalization is set to 
"nCounter".
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")

# Load data and positive control normalization
dat <- read_merge_rcc(list.files(example_data, full.names = TRUE))
dat <- normalize_pos_controls(dat)

# Remove endogenous genes that fail to reject the null hypothesis
# in a one-sided t test against negative control genes with p < 0.05.
dat <- remove_background(dat, mode = "t.test", pval = 0.05)

# Remove endogenous genes where fewer than 25\% of samples have an expression
# 2 standard deviations above the average negative control gene. Also, 
# subtract this background level (mean + 2*sd) from endogenous genes.
dat <- remove_background(dat, mode = "threshold", 
                         numSD = 2, proportionReq = 0.25, subtract = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_merge_rcc.R
\name{read_merge_rcc}
\alias{read_merge_rcc}
\title{Merge multiple .rcc files}
\usage{
read_merge_rcc(fileList, includeQC = FALSE, logfile = "")
}
\arguments{
\item{fileList}{a character vector of .rcc file names}

\item{includeQC}{include merged QC data (from the "Lane Attributes" part of 
file) in the output? Default FALSE}

\item{logfile}{a filename for the logfile (optional). If blank, will print 
warnings to screen.}
}
\value{
A list object including:
\item{exprs}{The expression matrix}
\item{dict}{The gene dictionary}
\item{qc}{QC metrics included in the .rcc files, if includeQC == TRUE}
}
\description{
Read in multiple .rcc files named in the fileList and merge the expression 
data. This step is conducted within processNanostringData.
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")

dat <- read_merge_rcc(list.files(example_data, full.names = TRUE))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/positiveQC.R
\name{positiveQC}
\alias{positiveQC}
\title{Calculate positive control statistics}
\usage{
positiveQC(ns, samples = NULL, expected = NULL)
}
\arguments{
\item{ns}{NanoString data, processed by `processNanostringData` with
normalization set to 'none' or with output.format set to 'list'.}

\item{samples}{A subset of samples to analyze (either a vector of sample 
names, or column indexes). If NULL (default), will include all samples.}

\item{expected}{The expected values of each positive control gene, as a 
numeric vector. These are frequently provided by NanoString in the 'Name' 
field of the genes, in which case those values will be read automatically 
and this option can be left as NULL (the default).}
}
\value{
A list object containing:
\item{tab}{The table of positive control statistics, included the positive
scale factor and the R-squared value for the expected vs. measured counts}
\item{plt}{An object containing the positive control plots. This gets
cumbersome if there are lots of samples.}
}
\description{
Calculate the linearity and scale factors of positive control genes, and
plot the expected vs. observed counts for each sample.
}
\examples{
example_data <- system.file("extdata", "GSE117751_RAW", package = "NanoTube")
sample_data <- system.file("extdata", 
                           "GSE117751_sample_data.csv", 
                           package = "NanoTube")

# Process data first. Must be output as a "list" or without normalization to
# obtain positive control statistics
dat <- processNanostringData(example_data, 
                             sampleTab = sample_data, 
                             groupCol = "Sample_Diagnosis",
                             normalization = "nSolver", 
                             bgType = "t.test", 
                             bgPVal = 0.01,
                             output.format = "list")

# Generate positive QC metrics for all samples
posQC <- positiveQC(dat) 

# View positive QC table & plot
head(posQC$tab)
posQC$plt

# Plot for only the first three samples
posQC <- positiveQC(dat, samples = 1:3)
posQC$plt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gm_mean.R
\name{gm_mean}
\alias{gm_mean}
\title{Calculate the geometric mean}
\usage{
gm_mean(x, na.rm = TRUE)
}
\arguments{
\item{x}{A numeric vector}

\item{na.rm}{Logical (default TRUE). Should NA values be ignored in this 
calculation? If FALSE, a vector containing NA values will return a 
geometric mean of NA.}
}
\value{
The geometric mean
}
\description{
Calculates the geometric mean of a numeric vector
}
\examples{
gm_mean(c(1, 3, 5))
}
